#!/usr/bin/env python
## From http://code.activestate.com/recipes/578294/ (r1)

def enum(*sequential, **named):
    # Check for duplicate keys
    names = list(sequential)
    names.extend(named.keys())
    if len(set(names)) != len(names):
        raise KeyError('Cannot create enumeration with duplicate keys!')

    # Build property dict
    enums = dict(zip(sequential, range(len(sequential))), **named)
    if not enums:
        raise KeyError('Cannot create empty enumeration')

    if len(set(enums.values())) < len(enums):
        raise ValueError('Cannot create enumeration with duplicate values!')

    # Function to be called as fset/fdel
    def err_func(*args, **kwargs):
        raise AttributeError('Enumeration is immutable!')

    # function to be called as fget
    def getter(cls, val):
        return lambda cls: val

    # Create a base type
    t = type('enum', (object,), {})

    # Add properties to class by duck-punching
    for attr, val in enums.iteritems():
        setattr(t, attr, property(getter(t, val), err_func, err_func))

    # Return an instance of the new class
    return t()


from akamu.wheezy import WheezyCachingAdapterSetup
from akara import global_config, request, module_config, response

class requires_authority(object):
    """

    Used with WheezyRepozeWrapper and RepozeWrapper as innermost decorator of
    akara service to indicate that authentication is required and cause
    401 response if there is none.  The first argument is the
    message to send to the client

    @requires_authority('Not authorized')
    def akara_service(): pass

    """
    def __init__(self, message = 'Not authorized', noauth = False):
        self.message = message
        self.noauth  = noauth
    def __call__(self, func):
        def innerHandler(*args, **kwds):
            if 'REMOTE_USER' not in request.environ and not self.noauth:
                response.code = 401
                return self.message
            else:
                if '_' in kwds:
                    del kwds['_']
                return func(*args, **kwds)
        return innerHandler


# -*- encoding: utf-8 -*-
'''
Manage diglot filesytem from Akara configuration

Requires a configuration section, for example:

class diglot:
    rootPath       = '..'
    datasetName    = '..'
    graphUriFn     = functionName
    transforms4Dir = {
        u'.. FS directory ..' : u'.. FS XSLT path ..'
    }
'''

import akara
from akamu.diglot import Manager

def GetDiglotManager(graphUriFn=None):
    if graphUriFn is None:
        graphUriFn = akara.module_config().get('graphUriFn')
    rootPath       = akara.module_config().get('rootPath')
    datasetName    = akara.module_config().get('datasetName')
    transforms4Dir = akara.module_config().get('transforms4Dir',{})
    return Manager(
        rootPath,
        datasetName,
        graphUriFn=graphUriFn,
        transforms4Dir=transforms4Dir)


__author__ = 'chimezieogbuji'

import cgi, amara
from akamu.xslt import xslt_rest
from akara.services import simple_service
from cStringIO import StringIO
from amara.writers.struct import structwriter, E, NS, ROOT
from amara.lib import U
from akara import request
from akamu.diglot import Manager, Resource
from akamu.xslt   import TransformWithAkamuExtensions
from akamu.config.dataset import DestroyOrCreateDataset
from akamu.config.diglot import GetDiglotManager

SERVICE_ID = 'http://example.com/xslt_rest'

def MakeFoafGraph(name='Chimezie Ogbuji'):
    src    = StringIO()
    w = structwriter(indent=u"yes", stream=src)
    w.feed(
        ROOT(
            E(u'Patient',{ u'name' : U(name),
                           u'gender' : u'Male'}
            )
        )
    )
    return src.getvalue()

@simple_service('GET', SERVICE_ID, 'xslt_rest_get','application/rdf+xml')
@xslt_rest('test/foaf.xslt',source=MakeFoafGraph,srcIsFn=True)
def rest_service(name='Chimezie Ogbuji'): pass

@simple_service('POST', SERVICE_ID, 'xslt_rest_post','application/rdf+xml')
@xslt_rest('test/foaf.xslt',source=MakeFoafGraph,srcIsFn=True)
def rest_service_post(body, ctype): pass

@simple_service('GET', SERVICE_ID, 'diglot_extensions_basic_test')
def test_diglot_extensions_basic(rootPath):
    def TestGraphUriFn(path,fName):
        return 'http://example.com%s'%path.split('.')[0]

    DestroyOrCreateDataset('mysqlDataset')
    mgr = GetDiglotManager(TestGraphUriFn)
    rt = TransformWithAkamuExtensions(
        '<Root/>',
        open('test/diglot_extension_test1.xslt').read(),
        mgr)
    DestroyOrCreateDataset('mysqlDataset')
    doc = amara.parse(rt)
    assert doc.xml_select(
            '/Answer/sparql:sparql/sparql:boolean[text() = "true"]',
            prefixes={u'sparql' : u'http://www.w3.org/2005/sparql-results#'}
    )
    assert doc.xml_select('/Answer/Patient[@name = "Uche Ogbuji"]')
    assert doc.xml_select('/Answer/AfterChange/Patient[@name = "Chimezie Ogbuji"]')
    assert doc.xml_select('/Answer/AfterChange/FoundPatientViaExtensionFunction')
    return "Success"


__author__ = 'chimezieogbuji'

import cgi, inspect, re, hashlib
from cStringIO   import StringIO as c_string_io
from StringIO import StringIO as regular_string_io
from amara.xslt  import transform
from akara       import request, response, global_config
from amara.xslt.processor import processor as amara_processor
from amara.lib import inputsource
# from akamu.xslt.extensions import NS

FIREFOX_PATTERN = re.compile(r'15\..+')
IE_9_PATTERN    = re.compile(r'9.0.+')
IE_8_PATTERN    = re.compile(r'8.0.+')
OPERA_PATTERN   = re.compile(r'11.62.+')
SAFARI_PATTERN  = re.compile(r'5.1.3')
CHROME_PATTERN  = re.compile(r'20\..*')

#See: http://greenbytes.de/tech/tc/xslt/
CLIENT_SIDE_BROWSERS = [
    ('Firefox',FIREFOX_PATTERN),
    ('Microsoft Internet Explorer', IE_9_PATTERN),
    ('Microsoft Internet Explorer',IE_8_PATTERN),
    ('Opera',OPERA_PATTERN),
    ('Safari',SAFARI_PATTERN),
    ('Chrome',CHROME_PATTERN),
]

def ClientSideXSLTCapable(environ):
    import httpagentparser
    agentInfo = httpagentparser.detect(environ.get('HTTP_USER_AGENT', ''))
    browserInfo = agentInfo['browser']
    supported = filter(lambda (name,versionPattn):
                            browserInfo['name'] == name and
                            versionPattn.match(browserInfo['version']),
                       CLIENT_SIDE_BROWSERS)
    return bool(supported)

def TransformWithAkamuExtensions(src,xform,manager,params=None,baseUri=None):
    import warnings;warnings.warn("Akamu XSLT extensions are not fully supported!")
    baseUri = baseUri if baseUri else NS
    from amara.xslt.result import streamresult
    from amara.xpath.util import parameterize
    params = parameterize(params) if params else {}
    processor         = amara_processor(ignore_pis=True)#Processor.Processor()
    processor.manager = manager
    processor.registerExtensionModules(['akamu.xslt.extensions'])

    result = streamresult(StringIO())
    source = inputsource(src,baseUri)#InputSource.DefaultFactory.fromString(src,baseUri)
    isrc   = inputsource(xform,baseUri)#InputSource.DefaultFactory.fromString(xform,baseUri)
    processor.append_transform(isrc)
    proc.run(source, params, result)
    # processor.run(
    #     source,
    #     result=result,
    #     ignorePis=True,
    #     topLevelParams=params
    # )
    return result.getvalue()

NOOPXML = u'<Root/>'

class xslt_rest(object):
    """
    Decorator of Akara services that will cause all invokations to
    route HTTP (query or form) parameters into the transform as
    xslt parameters.  The source of the transform (a string) is given by applying
    a user-specified function against the parameters and
    the result of the transformation of this (using a user-specified
    transform) is used as the result of the service
    """
    def __init__(self,
                 transform,
                 source            = None,
                 argRemap          = None,
                 parameters        = None,
                 clientSide        = False,
                 srcIsFn           = False,
                 etag_result       = False,
                 kwdArgumentsToDel = None):
        self.etag_result       = etag_result
        self.clientSide        = clientSide
        self.argRemap          = argRemap if argRemap else {}
        self.transform         = transform
        self.params            = parameters if parameters else {}
        self.source            = source if source else None
        self.srcIsFn           = srcIsFn
        self.kwdArgumentsToDel = kwdArgumentsToDel if kwdArgumentsToDel else ['_']

    def setEtagToResultTreeHash(self, src):
        #If user specifies, set hash of result as ETag of response
        if 'wheezy.http.cache_policy' in request.environ:
            #Resuse any policy set by decorated akara service
            policy = request.environ['wheezy.http.cache_policy']
        else:
            #Set the policy so wheezy.http caching middle ware can use it
            policy = request.environ['wheezy.http.cache_profile'].cache_policy()
            request.environ['wheezy.http.cache_policy'] = policy
        policy.etag(hashlib.sha1(src).hexdigest())

    def __call__(self, func):

        def innerHandler(*args, **kwds):
            argNames = inspect.getargspec(func).args
            parameters = {}
            parameters.update(self.params)
            isaPost = len(args) == 2 and list(argNames) == ['body','ctype']
            if isaPost:
                #Parameters in POST body
                fields = cgi.FieldStorage(
                    fp=regular_string_io(args[0]),
                    environ=request.environ
                )
                for k in fields:
                    val = fields.getvalue(k)
                    if k in self.argRemap:
                        parameters[self.argRemap[k]] = val
                    else:
                        parameters[k] = val
            else:
                #parameters to service method are GET query arguments
                for idx,argName in enumerate(argNames):
                    if argName in self.argRemap:
                        parameters[self.argRemap[argName]] = args[idx] if len(args) > idx + 1 else kwds[argName]
                    elif len(args) > idx + 1 or argName in kwds:
                        parameters[argName] = args[idx] if len(args) > idx + 1 else kwds[argName]
                for k,v in kwds.items():
                    if k in self.argRemap:
                        parameters[self.argRemap[k]] = v
                    else:
                        parameters[k] = v

            #Route non-keyword and keyword arguments and their values to
            #XSLT parameters
            argInfo = inspect.getargspec(func)
            vargs    = argInfo.varargs
            keywords = argInfo.keywords

            #If the source is a function, then the parameters
            #are given to it, otherwise, to the decorated service function
            srcFn = self.source if self.srcIsFn else func

            if keywords is None and argInfo.defaults:
                keywords = argInfo.args[-len(argInfo.defaults):]
                vargs    = argInfo.args[:-len(argInfo.defaults)]

            for _arg in self.kwdArgumentsToDel:
                if _arg in kwds:
                    del kwds[_arg]

            if isaPost:
                src = srcFn(*args) if (
                    self.source is None or self.srcIsFn
                ) else self.source
            elif vargs and keywords:
                src = srcFn(*args, **kwds) if (
                    self.source is None or self.srcIsFn
                ) else self.source
            elif vargs:
                src = srcFn(*args) if (
                    self.source is None or self.srcIsFn
                ) else self.source
            elif keywords:
                src = srcFn(**kwds) if (
                    self.source is None or self.srcIsFn
                ) else self.source
            elif argInfo.args and parameters:
                src = srcFn(*map(lambda i:parameters[i],
                                 argInfo.args)) if (
                    self.source is None or self.srcIsFn
                ) else self.source
            else:
                src = srcFn() if (
                    self.source is None or self.srcIsFn
                ) else self.source
            isInfoResource = (isinstance(response.code,int) and
                              response.code == 200
                             ) or (isinstance(response.code,basestring) and
                                   response.code.lower()) == '200 ok'
            if not isInfoResource:
                #If response is not a 200 then we just return it (since we can't
                # be invoking an XSLT HTTP operation)
                if self.etag_result:
                    self.setEtagToResultTreeHash(src)
                return src
            else:
                if isinstance(src,tuple) and len(src)==2:
                    src,newParams = src
                    parameters.update(newParams)
                authenticatedUser = request.environ.get('REMOTE_USER')
                if authenticatedUser:
                    parameters[u'user'] = authenticatedUser
                elif u'user' in parameters:
                    del parameters[u'user']
                if self.clientSide:
                    from amara.bindery import parse
                    doc = parse(src)
                    pi = doc.xml_processing_instruction_factory(
                        u"xml-stylesheet",
                        u'href="%s" type="text/xsl"'%self.transform)
                    doc.xml_insert(0,pi)
                    if self.etag_result:
                        self.setEtagToResultTreeHash(src)
                    return doc
                else:
                    rtStream = regular_string_io()
                    #Unicde must be given as inefficient stream
                    #See http://docs.python.org/2/library/stringio.html#cStringIO.StringIO
                    #and limitation on akara transform function
                    # print src, type(src), self.transform
                    # src = regular_string_io(src) if isinstance(src,unicode) else c_string_io(src)
                    src = src.encode('utf-8') if isinstance(src,unicode) else src
                    transform(src,self.transform,params=parameters,output=rtStream)
                    rt = rtStream.getvalue()
                    if self.etag_result:
                        self.setEtagToResultTreeHash(rt)
                    return rt
        return innerHandler


__author__ = 'chimezieogbuji'

from cStringIO                 import StringIO
try:
    from Ft.Xml.Xslt               import XsltElement, ContentInfo, AttributeInfo
    from Ft.Xml.XPath              import Conversions
    from Ft.Xml.Domlette           import NonvalidatingReader
    from Ft.Xml.Xslt               import OutputParameters
    from Ft.Xml.Xslt.CopyOfElement import CopyNode
except ImportError:
    import warnings;warnings.warn("unable to import 4Suite, extensions not supported")
from akamu.config.dataset      import ConnectToDataset
from rdflib.Graph              import Graph, ConjunctiveGraph
from rdflib                    import plugin, URIRef

NS=u'tag:metacognitionllc.com,2012:AkamuXsltExtensions'

class SPARQLQueryElement(XsltElement):
    """
    This extension evaluates a SPARQL query either against the Akamu RDF data
    set(s) identified by name either within the targetGraph or across the entire
    dataset.  The schema is a path within the Akamu Diglot FS to an OWL or RDFS
    documentation of the vocabulary for use in optimizing queries over
    MySQL/layercake-python datasets

    The body of this element is a template; when the template is instantiated,
    it is processed as a SPARQL query string evaluated using the given
    extension attributes

    """
    content = ContentInfo.Template
    legalAttrs = {
        'targetGraph'      : AttributeInfo.StringAvt(default=''),
        'datasetName'      : AttributeInfo.StringAvt(default=''),
        'schema'           : AttributeInfo.StringAvt(default='')
    }

    def instantiate(self, context, processor):
        try:
            # Set the output to an RTF writer, which will create an RTF for
            # us.
            processor.pushResultTree(self.baseUri)

            # The template is manifested as children of the extension element
            # node.  Instantiate each in turn.
            for child in self.children:
                child.instantiate(context, processor)
        # You want to be sure you re-balance the stack even in case of
        # error.
        finally:
            # Retrieve the resulting RTF.
            result_rtf = processor.popResult()
            query=result_rtf.childNodes[0].nodeValue
            context.setProcessState(self)
            context.processorNss = self.namespaces
            targetGraph     = Conversions.StringValue(self._targetGraph.evaluate(context))
            datasetName     = Conversions.StringValue(self._datasetName.evaluate(context))
            schema          = self._schema.evaluate(context)

            store   = ConnectToDataset(datasetName)

            dataset = Graph(store,URIRef(targetGraph)) if targetGraph else ConjunctiveGraph(store)

            print "SPARQL: \n%s" % query
            if schema:
                pass
#                ontGraph=Graph().parse(StringIO(repo.fetchResource(schema).getContent()))
#                for litProp,resProp in ontGraph.query(OWL_PROPERTIES_QUERY,
#                    initNs={u'owl':OWL_NS}):
#                    if litProp:
#                        print "\t Registering literal property ", litProp
#                        dataset.store.literal_properties.add(litProp)
#                    if resProp:
#                        print "\t Registering resource property ", resProp
#                        dataset.store.resource_properties.add(resProp)
            try:
                rt  = dataset.query(
                            query.encode('utf-8'),
                            initNs=self.namespaces).serialize(format='xml')
                doc = NonvalidatingReader.parseString(
                        rt.encode('utf-8'),
                        'tag:ogbujic@ccf.org:2007:meaninglessURI')
#    doc = _dispatchSPARQLQuery(connection,
#                    storeKind,
#                    storeIdentifier,
#                    dataset,
#                    targetGraph,
#                    query,
#                    self.namespaces)
            except:
                #Error
                import traceback
                st = StringIO()
                traceback.print_exc(file=st)
                processor.xslMessage(st.getvalue())
                raise
            processor.writers[-1].copyNodes(doc.documentElement)

class CreateResourceElement(XsltElement):
    content = ContentInfo.Template
    legalAttrs = {
        'path'      : AttributeInfo.StringAvt(default=''),
        'literal-content' : AttributeInfo.YesNo(
            default='no',
            description=('If yes, treat the content literally, i.e. do not'
                         ' execute any XSLT instructions or extensions')),
        'method' : AttributeInfo.QNameAvt(),
        'version' : AttributeInfo.NMTokenAvt(),
        'encoding' : AttributeInfo.StringAvt(),
        'omit-xml-declaration' : AttributeInfo.YesNoAvt(),
        'standalone' : AttributeInfo.YesNoAvt(),
        'doctype-public' : AttributeInfo.StringAvt(),
        'doctype-system' : AttributeInfo.StringAvt(),
        'cdata-section-elements' : AttributeInfo.QNamesAvt(),
        'indent' : AttributeInfo.YesNoAvt(),
        'media-type' : AttributeInfo.StringAvt(),
    }

    doesSetup = True
    def setup(self):
        self._output_parameters = OutputParameters.OutputParameters()
        return

    def instantiate(self, context, processor):
        context.setProcessState(self)
        stream = StringIO()
        self._output_parameters.avtParse(self, context)
        processor.addHandler(self._output_parameters, stream)
        literal_content = self._literal_content
        if literal_content:
            for child in self.children:
                CopyNode(processor, child)
        else:
            self.processChildren(context, processor)
        processor.removeHandler()
        text = stream.getvalue()
        path = Conversions.StringValue(self._path.evaluate(context))
        processor.manager.createResource(path,text.encode('utf-8'))

class GetResourceContentElement(XsltElement):
    content = ContentInfo.Empty
    legalAttrs = {
        'path'      : AttributeInfo.StringAvt(default='')
    }
    def instantiate(self, context, processor):
        context.setProcessState(self)
        path = Conversions.StringValue(self._path.evaluate(context))
        content = processor.manager.getResource(path).getContent()
        doc = NonvalidatingReader.parseString(content,NS)
        processor.writers[-1].copyNodes(doc.documentElement)

def GetResourceFunction(context,path):
    content = context.processor.manager.getResource(path).getContent()
    doc = NonvalidatingReader.parseString(content,NS)
    return [ doc.documentElement ]

class UpdateResourceElement(XsltElement):
    legalAttrs = {
        'path'      : AttributeInfo.StringAvt(default=''),
        'literal-content' : AttributeInfo.YesNo(
            default='no',
            description=('If yes, treat the content literally, i.e. do not'
                         ' execute any XSLT instructions or extensions')),
        'method' : AttributeInfo.QNameAvt(),
        'version' : AttributeInfo.NMTokenAvt(),
        'encoding' : AttributeInfo.StringAvt(),
        'omit-xml-declaration' : AttributeInfo.YesNoAvt(),
        'standalone' : AttributeInfo.YesNoAvt(),
        'doctype-public' : AttributeInfo.StringAvt(),
        'doctype-system' : AttributeInfo.StringAvt(),
        'cdata-section-elements' : AttributeInfo.QNamesAvt(),
        'indent' : AttributeInfo.YesNoAvt(),
        'media-type' : AttributeInfo.StringAvt(),
        }

    doesSetup = True
    def setup(self):
        self._output_parameters = OutputParameters.OutputParameters()
        return

    def instantiate(self, context, processor):
        context.setProcessState(self)
        stream = StringIO()
        self._output_parameters.avtParse(self, context)
        processor.addHandler(self._output_parameters, stream)
        literal_content = self._literal_content
        if literal_content:
            for child in self.children:
                CopyNode(processor, child)
        else:
            self.processChildren(context, processor)
        processor.removeHandler()
        text = stream.getvalue()
        path = Conversions.StringValue(self._path.evaluate(context))
        res = processor.manager.getResource(path)
        res.update(text)

ExtFunctions = {
#    (NS, 'sparql-query') : SPARQLQueryFunction,
    (NS, 'get-resource') : GetResourceFunction
}

ExtElements = {
    (NS, 'sparql-query')    : SPARQLQueryElement,
    (NS, 'create-resource') : CreateResourceElement,
    (NS, 'get-resource')    : GetResourceContentElement,
    (NS, 'update-resource') : UpdateResourceElement
}


__author__ = 'chimezieogbuji'

import os, imp, random, string, datetime, hashlib
from cStringIO import StringIO
from akamu.config.diglot import GetDiglotManager
from akamu.diglot import layercake_mimetypes, XML_MT, GetFNameFromPath
from wsgiref.util import shift_path_info, request_uri
from akara import request
from amara.lib import U, inputsource
from webob import Request
from amara.xslt  import transform
from amara.bindery import parse
from amara.xupdate import reader, XUpdateError, apply_xupdate
from rdflib import URIRef
from rdflib.Graph import Graph
from akamu.config.dataset import ConnectToDataset
from akamu.wheezy import WheezyCachingAdapterSetup

GRDDL_NS   = u'http://www.w3.org/2003/g/data-view#'
XUPDATE_NS = u'http://www.xmldb.org/xupdate'

XSLT_MT    = u'application/xslt+xml'

def updatePolicy4File(filePath,policy):
    mtime_dt = datetime.datetime.fromtimestamp(os.path.getmtime(filePath))
    policy.last_modified(mtime_dt)
    policy.etag(hashlib.sha1(open(filePath).read()).hexdigest())

def HandleGET(req,mgr,environ,root,start_response):
    path = request_uri(environ,0).split(root)[-1]
    res = mgr.getResource(path)
    content = res.getContent()
    requestedXML = XML_MT in req.accept
    requestedRDF = req.accept.best_match(list(layercake_mimetypes))
    if 'HTTP_ACCEPT' not in environ or (requestedXML and not requestedRDF):
        start_response("200 Ok",
            [("Content-Type", XML_MT),
             ("Content-Length", len(content))]
        )
        return content
    elif requestedRDF and requestedXML:
        #requested both RDF media types and POX
        doc=parse(content)
        docRoot = doc.xml_select('/*')[0]

        transformPath = os.path.join(root,mgr.findTransform(path)[1:])
        docRoot.xmlns_attributes[u'grddl'] = GRDDL_NS
        docRoot.xml_attributes[(GRDDL_NS,u'transformation')] = transformPath
        stream = StringIO()
        doc.xml_write(stream=stream)
        rt = stream.getvalue()
        start_response("200 Ok",
            [("Content-Type", XML_MT),
             ("Content-Length", len(rt))]
        )
        return rt
    elif requestedRDF:
        store = ConnectToDataset(mgr.datasetName)
        preferredMT = req.accept.best_match(layercake_mimetypes)
        graphUri = URIRef(
            mgr.graphUriFn(
                path,
                GetFNameFromPath(path))
        )
        rt = Graph(store,identifier=graphUri).serialize(layercake_mimetypes[preferredMT])
        start_response("200 Ok",
            [("Content-Type", preferredMT),
             ("Content-Length", len(rt))]
        )
        return rt
    else:
        raise

def HandlePATCH(req,mgr,environ,root,start_response):
    path = request_uri(environ,0).split(root)[-1]
    res = mgr.getResource(path)
    content = res.getContent()

    if req.content_type == XSLT_MT:
        newContent = transform(content,req.body)
        res.update(newContent,parameters=req.params)
        start_response("200 Ok",[])
        return ''
    else:
        payloadDoc = parse(req.body)
        if payloadDoc.xml_select('/xu:modifications',prefixes={u'xu' : XUPDATE_NS}):
            from amara.xupdate import reader, XUpdateError, apply_xupdate
            from amara.lib import inputsource
            source     = inputsource( content, 'source')
            xupdate    = inputsource(req.body, 'xupdate-source')
            newContent = apply_xupdate(source, xupdate)
            rt=StringIO()
            newContent.xml_write(stream=rt)
            res.update(rt.getvalue(),parameters=req.params)
            start_response("200 Ok",[])
            return ''
        else:
            rt = 'PATCH body must be XSLT (application/xslt+xml) or XUpdate'
            start_response("400 Bad Request",
                [("Content-Length",  len(rt))])
            return rt

def HandlePOST(req,mgr,diglotPath,base=None):
    targetContent = mgr.getResource(diglotPath).getContent()

    if req.content_type == XML_MT:
        #@TODO: handle parameters
        return transform(req.body,targetContent), 'application/xml+rdf'
    else:
        doc=parse(targetContent)
        payloadGraph = Graph().parse(
            StringIO(req.body),
            format   = layercake_mimetypes[req.content_type],
            publicID = URIRef(base) if base else None
        )
        for revXFormSrc in doc.xml_select(
            '/xsl:stylesheet/ggs:reverse_transform',prefixes={
                u'ggs': u'http://code.google.com/p/akamu/wiki/DiglotFileSystemProtocol#Bidirectional_transformations',
                u'xsl': u'http://www.w3.org/1999/XSL/Transform'}):
            fnCode = revXFormSrc.xml_select('string(text())').strip()
            module = imp.new_module('inverseMap')
            exec fnCode in module.__dict__
            return module.ReverseTransform(payloadGraph), XML_MT

def HandleDirectoryPOST(req,start_response,mgr,newPath,newDiglotPath):
    if req.content_type == XML_MT:
        mgr.createResource(newPath,req.body,parameters=req.params)
        start_response("201 Created",
            [("Location", newDiglotPath),
                ("Content-Length",  0)])
        return ''
    else:
        requestedRDF = req.accept.best_match(list(layercake_mimetypes))
        if not requestedRDF:
            rt = "Didn't provide an RDF Content-type header"
            start_response("400 Bad Request",
                [("Content-Length",  len(rt))])
            return rt
        else:
            base = req.GET.get('base')
            payloadGraph = Graph().parse(
                StringIO(req.body),
                format   = layercake_mimetypes[req.content_type],
                publicID = URIRef(base) if base else None
            )
            xform = mgr.findTransform(newPath)
            targetContent = mgr.getResource(xform).getContent()
            doc=parse(targetContent)

            revXForm = doc.xml_select(
                '/xsl:stylesheet/ggs:reverse_transform',prefixes={
                    u'ggs': u'http://code.google.com/p/akamu/wiki/DiglotFileSystemProtocol#Bidirectional_transformations',
                    u'xsl': u'http://www.w3.org/1999/XSL/Transform'})
            if revXForm:
                revXFormSrc = revXForm[0]
                fnCode = revXFormSrc.xml_select('string(text())').strip()
                module = imp.new_module('inverseMap')
                exec fnCode in module.__dict__
                xmlDoc = module.ReverseTransform(payloadGraph)
                mgr.createResource(newPath,xmlDoc,parameters=req.params)
                start_response("201 Created",
                    [("Location", newDiglotPath),
                        ("Content-Length",  0)])
                return ''
            else:
                rt = "Target XSLT doesn't have a reverse transform"
                start_response("400 Bad Request",
                [("Content-Length",  len(rt))])
                return rt

def random_filename(chars=string.hexdigits, length=16, prefix='',
                    suffix='', verify=True, attempts=10):
    """
    From - http://ltslashgt.com/2007/07/23/random-filenames/
    """
    for attempt in range(attempts):
        filename = ''.join([random.choice(chars) for i in range(length)])
        filename = prefix + filename + suffix
        if not verify or not os.path.exists(filename):
            return filename

class xforms_grddl(object):
    """
    Decorator for service for an XForms documents
    that manage Diglot resources (served from a
    mounted Diglot Filesystem Protocol instance)

    Can be bound for use with a particular XForm document or
    the document used can be provided as a parameter to the
    service invokation.
    """
    def __init__(self,
                 instanceId='diglot-resource',
                 submissionId='diglot-submission',
                 hostDocumentBase=None,
                 hostDocument=None,
                 diglotRoot=None,
                 hostDocumentParam='document',
                 instanceAttribute='src',
                 instanceParam='src'):
        self.submissionId       = submissionId
        self.diglotRoot         = diglotRoot
        self.hostDocumentBase   = hostDocumentBase
        self.instanceParam      = instanceParam
        self.instanceId         = instanceId
        self.instanceAttribute  = instanceAttribute
        self.hostDocument       = hostDocument
        self.hostDocumentParam  = hostDocumentParam

    def __call__(self, func):
        def innerHandler(*args, **kwds):
            req = Request(request.environ)

            xformDocument = req.params.get(self.hostDocumentParam,self.hostDocument)
            xformDocument = os.path.join(
                self.hostDocumentBase,
                xformDocument) if self.hostDocumentBase else xformDocument

            instancePath = req.params[self.instanceParam]
            instancePath = os.path.join(
                self.diglotRoot,
                instancePath) if self.diglotRoot else instancePath

            updateSrc =\
            """<?xml version="1.0"?>
            <xupdate:modifications
                version="1.0"
                xmlns:xhtml="http://www.w3.org/1999/xhtml"
                xmlns:xf="http://www.w3.org/2002/xforms"
                xmlns:xupdate="http://www.xmldb.org/xupdate">
                <xupdate:update select="/xhtml:html/xhtml:head/xf:model/xf:instance[@id = '%s']/@%s">%s</xupdate:update>
                <xupdate:update select="/xhtml:html/xhtml:head/xf:model/xf:submission[@id = '%s']/@resource">%s</xupdate:update>
            </xupdate:modifications>
            """%(
                    self.instanceId,
                    self.instanceAttribute,
                    instancePath,
                    self.submissionId,
                    instancePath
                )
            source      = inputsource(xformDocument, 'source')
            xupdate     = inputsource(updateSrc, 'xupdate-source')
            doc = apply_xupdate(source, xupdate)
            aStream = StringIO()
            doc.xml_write(stream=aStream)
            return aStream.getvalue()
        return innerHandler

class grddl_graphstore_resource(object):
    """
    Extension of SPARQL Graph Store Protocol for use with Diglot filesystem

    Payload are GRDDL source documents,
    GET requests without Accept headers or with an 'application/xml' Accept header
     returns the XML document, those with RDF media types and application/xml return
     GRDDL source documents with references to transformation for the directory, and
     Accept headers with RDF media types return the RDF faithful rendition of the
     Diglot resource.

     XML posted to transforms in the diglot system invoke the transformation of
     the document, returning the GRDDL result graph.  RDF posted to transforms
     with reverse mappings return the corresponding XML document

     XUpdate sent via HTTP PATCH request to diglot resources will be applied to them.
     Closed XSLT transforms sent via HTTP PATCH to diglot resources will also be applied,
     replacing them with the result

     HEAD requests, are the same as GET but without any returned content

     Mounts an implementation of the protocol at the specified root and using the
     given graphUriFn funciton for use with the manager

    """
    def __init__(self, root, graphUriFn, caching = False, cacheability = None):
        self.root         = root
        self.graphUriFn   = graphUriFn
        self.cachingSetup = WheezyCachingAdapterSetup(
            queries=['base'],
            environ=['HTTP_ACCEPT'],
            asFn = True
            ) if caching else None
        if cacheability:
            self.cachingSetup.name = 'wheezyApp'
        self.cacheability = cacheability if cacheability else 'public'

    def introspect_resource(self,req,mgr):
        diglotPath = req.path.split(self.root)[-1]
        isXslt = diglotPath in mgr.transforms4Dir.values()
        return diglotPath, isXslt

    def handleResourceCacheability(self,mgr,diglotPath,cacheability,environ):
        policy = environ['wheezy.http.HTTPCachePolicy'](cacheability)
        updatePolicy4File(
            mgr.getFullPath(diglotPath),
            policy
        )
        environ['wheezy.http.cache_policy']     = policy
        environ['wheezy.http.cache_dependency'] = diglotPath

    def invalidateResourceCache(self,cacheability,environ,diglotPath):
        policy = environ['wheezy.http.HTTPCachePolicy'](cacheability)
        environ['akamu.wheezy.invalidate'](diglotPath)
        environ['wheezy.http.cache_policy'] = policy

    def __call__(self, func):
        def innerHandler(environ, start_response):
            req = Request(environ)
            mgr = GetDiglotManager(self.graphUriFn)
            diglotPath, isXslt = self.introspect_resource(req,mgr)
            try:
                if req.method   in ['HEAD','GET']:
                    rt = HandleGET(req,mgr,environ,self.root,start_response)
                    if req.method == 'GET':
                        if self.cachingSetup:
                            self.handleResourceCacheability(
                                mgr,
                                diglotPath,
                                self.cacheability,
                                environ
                            )
                        return rt
                    else:
                        return ''
                elif req.method == 'PUT':
                    _path = mgr.getFullPath(diglotPath)
                    if not os.path.exists(_path):
                        mgr.createResource(diglotPath,req.body,parameters=req.params)
                        if self.cachingSetup: environ['wheezy.http.noCache'] = True
                        start_response("201 Created",[])
                    else:
                        mgr.getResource(diglotPath).update(req.body,parameters=req.params)
                        environ['wheezy.http.noCache'] = True
                        start_response("204 No Content",[])
                        if self.cachingSetup:
                            self.invalidateResourceCache(
                                self.cacheability,
                                environ,
                                diglotPath
                            )
                    return ''
                elif req.method == 'DELETE':
                    mgr.getResource(diglotPath).delete()
                    msg = '%s has been deleted'%diglotPath
                    start_response("204 No Content",
                        [("Content-Length",  len(msg))]
                    )
                    if self.cachingSetup:
                        self.invalidateResourceCache(
                            self.cacheability,
                            environ,
                            diglotPath
                        )
                        environ['wheezy.http.noCache'] = True
                    return msg
                elif req.method == 'PATCH':
                    rt = HandlePATCH(req,mgr,environ,self.root,start_response)
                    if self.cachingSetup:
                        self.invalidateResourceCache(
                            self.cacheability,
                            environ,
                            diglotPath
                        )
                        environ['wheezy.http.noCache'] = True
                    return rt
                elif req.method == 'POST' and isXslt:
                    rt,cont_type = HandlePOST(req,mgr,diglotPath,base=req.GET.get('base'))
                    start_response("200 Ok",
                        [("Content-Type"  , cont_type),
                         ("Content-Length",  len(rt))]
                    )
                    if self.cachingSetup: environ['wheezy.http.noCache'] = True
                    return rt
                elif req.method == 'POST' and os.path.isdir(mgr.getFullPath(diglotPath)):
                    randFileN = random_filename()+'.xml'
                    newPath       = os.path.join(diglotPath,randFileN)
                    newDiglotPath = os.path.join(self.root,diglotPath[1:],randFileN)
                    if self.cachingSetup: environ['wheezy.http.noCache'] = True
                    return HandleDirectoryPOST(
                        req,
                        start_response,
                        mgr,
                        newPath,
                        newDiglotPath
                    )
                else:
                    start_response("405 Method Not Allowed",[])
                    return "Method not allowed for this resource"
            except IOError, e:
                msg = str(e)#e.message
                start_response("404 Method Not Allowed",[("Content-Length",  len(msg))])
                return msg
        if self.cachingSetup:
            return self.cachingSetup(innerHandler)
        else:
            return innerHandler


import os
from cStringIO import StringIO
from akamu.config.dataset import ConnectToDataset, ReplaceGraph, ClearGraph
from amara.xslt  import transform
from rdflib.store import Store
from rdflib.Graph import Graph
from rdflib import URIRef

def GetFNameFromPath(path):
    return path.split('/')[-1]

layercake_mimetypes = {
    'application/rdf+xml' : 'xml',
    'text/n3'             : 'n3',
    'text/turtle'         : 'turtle',
    #'text/plain'          : 'nt'
}

layercake_parse_mimetypes = {
    'application/rdf+xml' : 'xml',
    'text/n3'             : 'n3',
    'text/turtle'         : 'n3',
    #'text/plain'          : 'nt'
}

XML_MT = 'application/xml'

class Manager(object):
    def __init__(self,root,datasetName=None,graphUriFn=None,transforms4Dir=None):
        self.root           = root
        self.datasetName    = datasetName
        self.graphUriFn     = graphUriFn
        self.transforms4Dir = transforms4Dir

    def hasResource(self,path):
        return os.path.exists(
            os.path.join(
                self.root,
                path[1:] if path[0] == '/' else path
            )
        )

    def getFullPath(self,path):
        return os.path.join(self.root,path[1:] if path[0] == '/' else path)

    def findTransform(self,path):
        parentDir = '/'.join(path.split('/')[:-1]) if path.find('/')+1 else '/'
        rootXform = self.transforms4Dir.get('/')
        xform     = self.transforms4Dir.get(parentDir)
        if xform:
            return xform
        elif len(path.split('/'))>1:
            return self.findTransform(parentDir)
        elif rootXform:
            return rootXform

    def synch(self,path=None,parameters=None):
        parameters = parameters if parameters else {}
        if path:
            graphUri = URIRef(self.graphUriFn(path,GetFNameFromPath(path)))
            params = {
                u'path'     : path,
                u'graph-uri': graphUri
            }
            params.update(parameters)

            xFormPath = self.findTransform(path)
            rt = transform(
                self.getResource(path).getContent(),
                self.getResource(xFormPath).getContent(),
                params=params
            )
            ReplaceGraph(
                self.datasetName,
                graphUri,
                StringIO(rt),
                format='xml',
                storeName=not isinstance(self.datasetName,Store),
                baseUri=graphUri
            )
        else:
            raise NotImplementedError("[..]")

    def getResource(self,path):
        return Resource(self,path)

    def deleteResource(self,path):
        self.getResource(path).delete()

    def createResource(self,path,content,parameters=None):
        parameters = parameters if parameters else {}
        res = Resource(self,path)
        res.update(content,parameters)
        return res

class Resource(object):
    def __init__(self,manager,path):
        self.manager = manager
        self.path    = path

    def delete(self):
        os.remove(self.manager.getFullPath(self.path))
        graphUri = URIRef(
            self.manager.graphUriFn(
                self.path,GetFNameFromPath(self.path))
        )
        ClearGraph(
            self.manager.datasetName,
            graphUri,
            storeName=not isinstance(self.manager.datasetName,Store))

    def update(self,content,parameters=None):
        parameters = parameters if parameters else {}
        f=open(self.manager.getFullPath(self.path),'w')
        f.write(content)
        f.close()
        self.manager.synch(self.path,parameters)

    def getContent(self,mediaType=None):
        if mediaType is None or mediaType == XML_MT:
            return open(self.manager.getFullPath(self.path)).read()
        else:
            store = ConnectToDataset(self.manager.datasetName)
            graphUri = URIRef(self.manager.graphUriFn(self.path,GetFNameFromPath(path)))
            graph = Graph(store,identifier=graphUri)
            return graph.serialize(format=layercake_mimetypes[mediaType])


__author__ = 'chimezieogbuji'

import os, string, random, pyparsing, time, re
from cStringIO            import StringIO
from webob                import Request
try:
    from Triclops.Server  import WsgiApplication, SD_FORMATS, MIME_SERIALIZATIONS
except ImportError: pass
from akamu.config.dataset import ConnectToDataset, GetGraphStoreForProtocol, ConfigureTriclops, GetExternalGraphStoreURL
from akamu.diglot         import layercake_mimetypes, XML_MT, layercake_parse_mimetypes
from amara.lib            import iri
from amara.xslt           import transform
from akara                import request
from wsgiref.util         import shift_path_info, request_uri
from rdflib.Graph         import Graph,ConjunctiveGraph
from rdflib.sparql        import parser as sparql_parser
from rdflib               import OWL, RDF, RDFS, URIRef, BNode, Namespace, Literal
from akamu.util           import enum
from amara.lib.iri        import uri_to_os_path
RESULT_FORMAT = enum(CSV='CSV',TSV='TSV',JSON='JSON',XML='XML')

DefaultGraph_NS = Namespace('tag:metacognition.info,2012:DefaultGraphs#')

def random_filename(chars=string.hexdigits, length=16, prefix='',
                    suffix='', verify=True, attempts=10):
    """
    From - http://ltslashgt.com/2007/07/23/random-filenames/
    """
    for attempt in range(attempts):
        filename = ''.join([random.choice(chars) for i in range(length)])
        filename = prefix + filename + suffix
        if not verify or not os.path.exists(filename):
            return filename

class NoEmptyGraphSupport(Exception):
    def __init__(self):
        super(NoEmptyGraphSupport, self).__init__(
            "Implementation does not support empty graphs"
        )

def RequestedGraphContent(req,store,datasetName):
    graphParamValue = req.params.get('graph')
    if 'default' in req.params:
        graph = Graph(identifier=DefaultGraph_NS[datasetName],store=store)
    else:
        graphIri        = graphParamValue if graphParamValue else req.url
        graph           = Graph(identifier=URIRef(graphIri),store=store)
    return graph

def HandleGET(req,environ,start_response,store,datasetName):
    graph           = RequestedGraphContent(req,store,datasetName)
    if not graph and 'default' not in req.params:
        raise NoEmptyGraphSupport()

    requestedRDF = req.accept.best_match(list(layercake_mimetypes))
    if 'HTTP_ACCEPT' not in environ or not requestedRDF:
        requestedMT = 'application/rdf+xml'
        format      = 'pretty-xml'
    elif requestedRDF:
        preferredMT = req.accept.best_match(layercake_mimetypes)
        requestedMT = preferredMT
        format = layercake_mimetypes[preferredMT]
    content = graph.serialize(format=format)
    start_response("200 Ok",
        [("Content-Type"  , requestedMT),
         ("Content-Length", len(content))]
    )
    return content

def HandlePUT(req,start_response,store,datasetName):
    graph           = RequestedGraphContent(req,store,datasetName)
    nonexistent = not graph and 'default' not in req.params
    if not req.content_type:
        rt = "Didn't provide an RDF Content-type header"
        start_response("400 Bad Request",
            [("Content-Length",  len(rt))])
        return rt
    else:
        format = layercake_parse_mimetypes.get(
            req.content_type,
            layercake_mimetypes[req.content_type])
        try:
            payloadGraph = Graph().parse(StringIO(req.body),format = format)
        except Exception, e:
            rt = e.message
            start_response("400 Bad Request",
                [("Content-Length",  len(rt))])
            return rt
    print req.url, graph.identifier, req.params,bool(payloadGraph)
    if payloadGraph:
        graph.remove((None,None,None))
        for s,p,o in payloadGraph:
            graph.add((s,p,o))
        store.commit()
        if nonexistent:
            start_response("201 Created",[])
        else:
            start_response("204 No Content",[])
        return ""
    else:
        #Empty graph
#        raise NoEmptyGraphSupport()
        start_response("200 Ok",[])
        return "NOOP: server doesn't support empty graphs"

def HandleDELETE(req,start_response,store,datasetName):
    graph           = RequestedGraphContent(req,store,datasetName)
    if not graph and 'default' not in req.params:
        raise NoEmptyGraphSupport()
    else:
        graph.remove((None,None,None))
        store.commit()
        start_response("200 Ok",[])
        return ""

def handleTrailingSlash(url,strip=True):
    if strip:
        return url if url[-1]!='/' else url[:-1]
    else:
        return url+'/' if url[-1]!='/' else url

def HandlePOST(req,start_response,store,graphStore,externalGS,datasetName):
    graph           = RequestedGraphContent(req,store,datasetName)

    if not req.content_type:
        rt = "Didn't provide an RDF Content-type header"
        start_response("400 Bad Request",
            [("Content-Length",  len(rt))])
        return rt
    if handleTrailingSlash(req.url) == handleTrailingSlash(graphStore):
        #If the request IRI identifies the underlying Graph Store, the origin
        #server MUST create a new RDF graph comprised of the statements in
        #the RDF payload and return a designated graph IRI associated with
        #the new graph. The new graph IRI should be specified in the Location
        #HTTP header along with a 201 Created code and be different from the
        #request IRI.
        new_filename = random_filename(suffix=req.params.get('suffix',''))

        new_location = iri.absolutize(
            new_filename,
            handleTrailingSlash(graphStore,strip=False)
        )
        new_location = URIRef(new_location)

        external_new_location = iri.absolutize(
            new_filename,
            handleTrailingSlash(
                externalGS if externalGS else graphStore,
                strip=False)
        )

        try:
            Graph(identifier=new_location,store=store).parse(
                StringIO(req.body),
                format   = layercake_parse_mimetypes[req.content_type]
            )
        except Exception, e:
            rt = e.message
            start_response("400 Bad Request",
                [("Content-Length",  len(rt))])
            return rt
        store.commit()
        start_response("201 Created",
            [("Location", external_new_location),
             ("Content-Length",  0)])
        return ''
    else:
        toAdd    = []
        canMerge = True
        if req.content_type == 'multipart/form-data':
            import cgi
            form = cgi.FieldStorage(
                fp=StringIO(req.body),
                environ=request.environ
            )
            try:
                for multipartEntry in form:
                    for triple in Graph().parse(
                        StringIO(form.getvalue(multipartEntry)),
                        format   = layercake_parse_mimetypes[form[multipartEntry].type]
                    ):
                        s,p,o = triple
                        if triple not in graph:
                            toAdd.append((s,p,o,graph))
                        elif [term for term in triple if isinstance(term,BNode)]:
                            #Blank node in common is shared, no support for this currently
                            #See: http://www.w3.org/TR/rdf-mt/#defmerge
                            canMerge = False
                            break
            except Exception, e:
                rt = str(e)
                start_response("400 Bad Request",
                    [("Content-Length",  len(rt))])
                return rt
        else:
            try:
                for triple in Graph().parse(
                    StringIO(req.body),
                    format   = layercake_parse_mimetypes[req.content_type]
                ):
                    s,p,o = triple
                    if triple not in graph:
                        toAdd.append((s,p,o,graph))
                    elif [term for term in triple if isinstance(term,BNode)]:
                        #Blank node in common is shared, no support for this currently
                        #See: http://www.w3.org/TR/rdf-mt/#defmerge
                        canMerge = False
                        break
            except Exception, e:
                rt = str(e)
                start_response("400 Bad Request",
                    [("Content-Length",  len(rt))])
                return rt
        if not canMerge:
            rt = "Merge involving shared blank nodes not supported"
            start_response("409 Conflict",
                [("Content-Length",  len(rt))])
            return rt
        else:
            graph.addN(toAdd)
            store.commit()
            start_response("200 Ok",[])
            return ''

class graph_store_protocol(object):
    """

    """
    def __init__(self):
        self.datasetName,self.gs_url = GetGraphStoreForProtocol()
        self.store                   = ConnectToDataset(self.datasetName)
        self.external_gs_url         = GetExternalGraphStoreURL()

    def __call__(self, func):
        def innerHandler(environ, start_response):
            req = Request(environ)
            try:
                if req.method   in ['HEAD','GET']:
                    rt = HandleGET(req,environ,start_response,self.store,self.datasetName)
                    if req.method == 'GET':
                        return rt
                    else:
                        return ''
                elif req.method == 'PUT':
                    return HandlePUT(req,start_response,self.store,self.datasetName)
                elif req.method == 'DELETE':
                    return HandleDELETE(req,start_response,self.store,self.datasetName)
                elif req.method == 'PATCH':
                    rt="PATCH not supported"
                    start_response("405 Method Not Allowed",
                        [("Content-Length",  len(rt))]
                    )
                    return rt
                elif req.method == 'POST':
                    return HandlePOST(
                        req,
                        start_response,
                        self.store,
                        self.gs_url,
                        self.external_gs_url,
                        self.datasetName)
                else:
                    start_response("405 Method Not Allowed",[])
                    return "Method not allowed for this resource"
            except NoEmptyGraphSupport, e:
                rt = "Implementation does not support empty graphs"
                start_response("404 Method Not Allowed",
                    [("Content-Length",  len(rt))])
                return rt
            except NotImplementedError, e:
                raise e
        return innerHandler

def GetResultFormats(results,xslt_dir,result_format=RESULT_FORMAT.XML):
    query_results = results.serialize(format='xml')

    if result_format == RESULT_FORMAT.JSON:
        serialization = transform(
            query_results,
            os.path.join(xslt_dir,'sparqlxml2json.xsl'),
            params={ u'ignore-bnode' : True }
        )
    elif result_format == RESULT_FORMAT.TSV:
        serialization  = transform(
            query_results,
            os.path.join(xslt_dir,'xml-to-csv-tsv.xslt'),
            params={u'standard':True}
        )
    else:
        serialization  = transform(
            query_results,
            os.path.join(xslt_dir,'xml-to-csv-tsv.xslt'),
            params={u'standard':True,u'tsv':False}
        )
    return serialization

class sparql_rdf_protocol(object):
    """
    Prepares a Triclops WSGI application for use to wrap
     the Akara via the 'wsgi_wrapper' keyword argument of
     @simple_service and @service

    See: http://code.google.com/p/python-dlp/wiki/Triclops
    """
    def __init__(self, root, datasetName):
        self.nsBindings    = {  u'owl' :OWL.OWLNS,
                                u'rdf' :RDF.RDFNS,
                                u'rdfs':RDFS.RDFSNS}
        self.litProps = set()
        self.resProps = set()
        self.root               = root
        self.datasetName        = datasetName
        self.conf               = ConfigureTriclops(
                                    self.datasetName,
                                    self.nsBindings,
                                    self.litProps,
                                    self.resProps)
        self.conf['endpoint']   = self.root

    def __call__(self, func):

        def innerHandler(environ, start_response):
            # @@@ Core Application @@@
            app = WsgiApplication(
                self.conf,
                self.nsBindings,
                [],
                self.litProps,
                self.resProps,
                Graph(),
                Graph(),
                set(),
                Graph())

            req = Request(environ)
            d                 = req.params
            query             = d.get('query')
            ticket            = d.get('ticket')
            default_graph_uri = d.get('default-graph-uri')
            rtFormat          = d.get('resultFormat')

            if 'query' in d and len(filter(lambda i:i == 'query',d))>1:
                rt = "Malformed SPARQL Query: query parameter provided twice"
                status = '400 Bad Request'
                response_headers = [('Content-type','text/plain'),
                                    ('Content-Length',
                                     len(rt))]
                start_response(status,response_headers)
                return rt

            if req.method == 'POST':
                if req.content_type == 'application/sparql-query':
                    query = req.body
                elif req.content_type == 'application/x-www-form-urlencoded':
                    query = req.POST.get('query')

            print "## Query ##\n", query, "\n###########"
            print "Default graph uri ", default_graph_uri
            requestedFormat = environ.get('HTTP_ACCEPT','application/rdf+xml')

            if req.method == 'POST':
                assert query,"POST can only take an encoded query or a query in the body"
            elif req.method == 'GET' and not query:
                if requestedFormat not in SD_FORMATS:
                    requestedFormat = 'application/rdf+xml'
                if app.ignoreQueryDataset:
                    targetGraph = app.buildGraph(default_graph_uri)
                else:
                    targetGraph = app.buildGraph(default_graph_uri=None)

                sdGraph = Graph()

                SD_NS  = Namespace('http://www.w3.org/ns/sparql-service-description#')
                SCOVO  = Namespace('http://purl.org/NET/scovo#')
                VOID   = Namespace('http://rdfs.org/ns/void#')
                FORMAT = Namespace('http://www.w3.org/ns/formats/')

                sdGraph.bind(u'sd',SD_NS)
                sdGraph.bind(u'scovo',SCOVO)
                sdGraph.bind(u'void',VOID)
                sdGraph.bind(u'format',FORMAT)

                service     = BNode()
                datasetNode = BNode()
                if app.endpointURL:
                    sdGraph.add((service,SD_NS.endpoint,URIRef(app.endpointURL)))
                sdGraph.add((service,SD_NS.supportedLanguage        ,SD_NS.SPARQL10Query))
                sdGraph.add((service,RDF.type                       ,SD_NS.Service))
                sdGraph.add((service,SD_NS.defaultDatasetDescription,datasetNode))
                sdGraph.add((service,SD_NS.resultFormat,FORMAT['SPARQL_Results_XML']))
                sdGraph.add((datasetNode,RDF.type,SD_NS.Dataset))

                for graph in targetGraph.store.contexts():
                    graphNode  = BNode()
                    graphNode2 = BNode()
                    sdGraph.add((datasetNode,SD_NS.namedGraph,graphNode))
                    sdGraph.add((graphNode,SD_NS.name,URIRef(graph.identifier)))
                    sdGraph.add((graphNode,SD_NS.graph,graphNode2))
                    sdGraph.add((graphNode,RDF.type,SD_NS.NamedGraph))
                    sdGraph.add((graphNode2,RDF.type,SD_NS.Graph))
                    noTriples = Literal(len(graph))
                    sdGraph.add((graphNode2,VOID.triples,noTriples))
                doc = sdGraph.serialize(
                    format=MIME_SERIALIZATIONS[requestedFormat])
                status = '200 OK'
                response_headers = [
                    ('Content-type'  , requestedFormat),
                    ('Content-Length', len(doc))
                ]
                start_response(status,response_headers)
                return doc
            else:
                assert req.method == 'GET',"Either POST or GET method!"
            if app.ignoreQueryDataset:
                app.targetGraph = app.buildGraph(default_graph_uri)
            else:
                app.targetGraph = app.buildGraph(default_graph_uri=None)

            for pref,nsUri in app.nsBindings.items():
                app.targetGraph.bind(pref,nsUri)

            origQuery = query
            describePattern=re.compile(r'DESCRIBE\s+\<(?P<iri>[^\>]+)\>',re.DOTALL)
            describeQueryMatch = describePattern.match(query)
            if describeQueryMatch:
                iri=URIRef(describeQueryMatch.group('iri'))
                g=Graph()
                for p,u in app.targetGraph.namespaces():
                    g.bind(p,u)
                for t in app.targetGraph.triples((None,None,iri)):
                    g.add(t)
                for t in app.targetGraph.triples((iri,None,None)):
                    g.add(t)
                rt=g.serialize(format='pretty-xml')
                status = '200 OK'
                response_headers = [('Content-type','application/rdf+xml'),
                                    ('Content-Length',
                                     len(rt))]
                start_response(status,response_headers)
                return rt
            try:
                query=sparql_parser.parse(query)
            except pyparsing.ParseException, e:
                rt = "Malformed SPARQL Query: %s"%repr(e)
                status = '400 Bad Request'
                response_headers = [('Content-type','text/plain'),
                                    ('Content-Length',
                                     len(rt))]
                start_response(status,response_headers)
                return rt

            start = time.time()

            if app.ignoreBase and hasattr(query,'prolog') and query.prolog:
                query.prolog.baseDeclaration=None
            if app.ignoreQueryDataset and hasattr(query.query,'dataSets') and query.query.dataSets:
                print "Ignoring query-specified datasets: ", query.query.dataSets
                query.query.dataSets = []

            if not app.proxy and ticket:
                #Add entry for current thread in ticket -> thread id lookup
                global ticketLookup
                ticketLookup[ticket]=app.targetGraph.store._db.thread_id()

                #Run the actual query
            rt = app.targetGraph.query(origQuery,
                initNs=app.nsBindings,
                DEBUG=app.debugQuery,
                parsedQuery=query)
            print "Time to execute SPARQL query: ", time.time() - start
            qRT = rt.serialize(format='xml')
            app.targetGraph.close()
            print "Time to execute and seralize SPARQL query: ", time.time() - start
            print "# of bindings: ", rt.noAnswers

            if rtFormat in ['xml','csv'] or not rtFormat:
#                from amara.bindery import parse
#                doc = parse(src)
#                pi = doc.xml_processing_instruction_factory(
#                    u"xml-stylesheet",
#                    u'href="%s" type="text/xsl"'%self.transform)
#                doc.xml_insert(0,pi)
#                return doc
                rt = qRT
                imt='application/sparql-results+xml'
#                rtDoc = NonvalidatingReader.parseString(qRT,
#                    'tag:nobody@nowhere:2007:meaninglessURI')
#                stylesheetPath = rtFormat == 'xml' and '/xslt/xml-to-html.xslt' or '/xslt/xml-to-csv.xslt'
#                imt='application/xml'
#
#                pi = rtDoc.createProcessingInstruction("xml-stylesheet",
#                    "type='text/xml' href='%s'"%stylesheetPath)
#                #Add a stylesheet instruction to direct browsers how to render the result document
#                rtDoc.insertBefore(pi, rtDoc.documentElement)
#                out = StringIO()
#                PrettyPrint(rtDoc, stream=out)
#                rt = out.getvalue()
            elif rtFormat == 'csv-pure':
                imt='text/plain'
                rt=app.csvProcessor.run(InputSource.DefaultFactory.fromString(qRT))

            status = '200 OK'
            response_headers = [('Content-type',imt),
                                ('Content-Length',len(rt))]
            start_response(status, response_headers)
            return rt
        return innerHandler




__author__ = 'chimezieogbuji'



import hashlib
from wheezy.http.cache import etag_md5crc32
from wheezy.caching.patterns import Cached
from wheezy.http.middleware import http_cache_middleware_factory
from wheezy.http.middleware import environ_cache_adapter_middleware_factory
from wheezy.http.middleware import wsgi_adapter_middleware_factory
from wheezy.caching.memcache import MemcachedClient
from wheezy.http import bootstrap_http_defaults, CacheProfile, WSGIApplication
from datetime import timedelta

class memcached(object):
    """
    Decorator that uses memcache to cache response to
    calling the function where the key is based on
    the values of the arguments given

    modified from http://www.zieglergasse.at/blog/2011/python/memcached-decorator-for-python/
    """
    def __init__(self,memcache_socket):
        self.memcache_socket = memcache_socket

    def __call__(self, f):

        def newfn(*args, **kwargs):
            mc = MemcachedClient([self.memcache_socket], debug=0)
            # generate md5 out of args and function
            m = hashlib.md5()
            margs = [x.__repr__() for x in args]
            mkwargs = [x.__repr__() for x in kwargs.values()]
            map(m.update, margs + mkwargs)
            m.update(f.__name__)
            m.update(f.__class__.__name__)
            key = m.hexdigest()

            value = mc.get(key)
            if value:
                return value
            else:
                value = f(*args, **kwargs)
                mc.set(key, value, 60)
                return value
        return newfn


def normalize_to_list(item):
    if item is None:
        return []
    else:
        return item if isinstance(item,list) else [item]

class WheezyCachingAdapterSetup(object):
    def __init__(self,
                 cache_location='public',
                 static_dependency = None,
                 queries = None,
                 ttl=15,
                 debug=False,
                 max_age=None,
                 memcache_socket = 'unix:/tmp/memcached.sock'):
        """
        Used as an Akara WSGI wrapper, i.e.:
        @simple_service(...,wsgi_wrapper=WheezyCachingAdapterSetup( ... ))

        Produces a wheezy.http.WSGIApplication that implements HTTP caching

        cache_location is used for the CacheProfile
        ( see: http://pythonhosted.org/wheezy.http/userguide.html#cache-location )

        static_dependency indicates the name of a master_key for a CacheDependency associated
        with the wrapped Akara service
        ( see: http://pythonhosted.org/wheezy.caching/userguide.html#cachedependency )

        queries is a list of query key's that will be used to compose the key for caching
        responses to requests

        ttl is the time to live for the content  (server) caching in seconds

        IF specified, max_age is used on the policy to set the corresponding
        HTTP cache control header (an indication to accept a response whose age
        is no greater than the specified time in seconds)

        memcache_socket is the memcache socket to use for content caching purposes

        Within an Akara service decorated in this way, a wheezy.http.HTTPCachePolicy instance can be created:

            profile = request.environ['wheezy.http.cache_profile']
            policy = profile.cache_policy()

        And then its various methods can be used to control cache-specific HTTP headers
        of the response:
            See: http://packages.python.org/wheezy.http/userguide.html#cache-policy

        Then request.environ['wheezy.http.cache_policy'] needs to be set to the policy:

            request.environ['wheezy.http.cache_policy'] = policy

        As an alternative to providing a static dependency name via the static_dependency
        keyword argument, a dependency with a dynamic master key can be provided via:

            request.environ['wheezy.http.cache_dependency'] = ['.. cache name ..', ..., ]

        Cache can be invalidated by (dependency) name via:

            request.environ['akamu.wheezy.invalidate']('..cache name..')

        """
        assert cache_location in ['none','server','client','public']
        self.cache              = MemcachedClient([memcache_socket])
        self.cache_location     = cache_location
        self.debug              = debug
        self.cached             = Cached(self.cache, time=ttl)
        self.static_dependency  = static_dependency
        self.max_age            = max_age
        self.cache_profile = CacheProfile(
            cache_location,
            vary_query=queries,
            enabled=True,
            etag_func=etag_md5crc32,
            duration=timedelta(seconds=ttl))

    def __call__(self,akara_application):
        """
        Called by Akara to provide the akara application
        as a WSGI application to be 'wrapped'

        Returns a wsgi_application that wraps the akara service and facilitating
        the use of WSGI environ variables for http content caching middleware.
        """
        self.akara_application = akara_application

        def wsgi_wrapper(environ, start_response):
            if self.cache_location != 'none':
                environ['wheezy.http.cache_profile'] = self.cache_profile
            def InvalidateCacheViaDependency(cacheName):
                from wheezy.caching.dependency import CacheDependency
                dependency = CacheDependency(
                    self.cache,
                    # namespace=self.cache_profile.namespace
                )
                if self.debug:
                    print "###","Invalidating cache: ", cacheName,"###"
                dependency.delete(cacheName)
                # self.cached.delete(cacheName)

            #Provide hook for cache dependency invalidation to akara service
            environ['akamu.wheezy.invalidate']     = InvalidateCacheViaDependency

            if self.debug:
                print "###","Calling akara application from wheezy.http","###"

            rt = akara_application(environ, start_response)
            if 'wheezy.http.cache_dependency' in environ:
                item = environ['wheezy.http.cache_dependency']
                environ['wheezy.http.cache_dependency'] = normalize_to_list(item)
                if self.debug:
                    print "###","Dependency key(s): ", environ['wheezy.http.cache_dependency'],"###"
            elif self.static_dependency:
                environ['wheezy.http.cache_dependency'
                ] = normalize_to_list(self.static_dependency)
                if self.debug:
                    print "###","Dependency key(s): ", self.static_dependency,"###"
            if self.max_age is not None:
                if self.debug:
                    print "###","Setting max_age and etag via function","###"
                policy = self.cache_profile.cache_policy()
                policy.max_age(self.max_age)
                policy.etag(self.cache_profile.etag_func(rt))
            return rt

        return WSGIApplication([
                bootstrap_http_defaults,
                http_cache_middleware_factory,
                environ_cache_adapter_middleware_factory,
                wsgi_adapter_middleware_factory
            ],
            {
                 'wsgi_app' : wsgi_wrapper,
                'http_cache': self.cache
            }
        )


__author__ = 'chimezieogbuji'

from akara.services import simple_service, service
from akamu.protocol.grddlstore import grddl_graphstore_resource

SERVICE_ID = 'http://code.google.com/p/akamu/wiki/DiglotFileSystemProtocol'

def TestGraphUriFn(path,fName):
    return 'http://example.com%s'%path.split('.')[0]

def ReverseTransform(graph):
    from rdflib import Namespace, RDF
    from amara.writers.struct import structwriter, E, ROOT
    from cStringIO import StringIO
    FOAF = Namespace("http://xmlns.com/foaf/0.1/")
    V    = Namespace("http://www.w3.org/2006/vcard/ns#")

    src = StringIO()
    w   = structwriter(indent=u"yes", stream=src)
    def attributes(personUri):
        attr = {}
        for _name in graph.query(
            "SELECT ?name [] a foaf:Person; foaf:businessCard [ v:fn ?name ]",
            initNs = { u'foaf' : FOAF, u'v' : V }
        ):
            attr[u'name'] = _name
        return attr
    for person in graph.subjects(RDF.type,FOAF.Person):
        w.feed(ROOT(E(u'Patient',attributes(person))))
    return src.getvalue()

@service(SERVICE_ID, 'diglot')
@grddl_graphstore_resource('/diglot', TestGraphUriFn, caching = False)
def grddl_graphstore_protocol(): pass


__author__ = 'chimezieogbuji'

import cgi, amara
from akara.services import simple_service
from cStringIO import StringIO
from amara.writers.struct import structwriter, E, NS, ROOT
from amara.lib import U
from akara import request
from akamu.wheezy import WheezyCachingAdapterSetup
from akara import request, response

XHTML_IMT  = 'application/xhtml+xml'
XML_IMT    = 'application/xml'
HTML_IMT   = 'text/html'
XHTML_NS   = u'http://www.w3.org/1999/xhtml'
SERVICE_ID = 'http://example.com/wheezy'

@simple_service('GET',
                SERVICE_ID,
                'wheezy.test',
                XHTML_IMT,
                wsgi_wrapper=WheezyCachingAdapterSetup(dependency='xhtmlContent'))
def wheezy_service():
    policy = request.environ['wheezy.http.HTTPCachePolicy']('public')

    policy.etag('12345')
    request.environ['wheezy.http.cache_policy']  = policy
    src    = StringIO()
    w = structwriter(indent=u"yes", stream=src)
    w.feed(
        ROOT(
            E((XHTML_NS,u'html'),
                E((XHTML_NS,u'body'),
                    E((XHTML_NS,u'p'),u'Testing')
                )
            )
        )
    )
    return src.getvalue()

@simple_service('GET',
                SERVICE_ID,
                'wheezy.test.clear_cache',
                XHTML_IMT,
                wsgi_wrapper=WheezyCachingAdapterSetup(noCache=True))
def wheezy_service2():
    request.environ['akamu.wheezy.invalidate']('xhtmlContent')
    return "Nothing"


import inspect
from webob import Request
from akara import request, response, global_config, registry
from akamu.config.dataset import ConnectToDataset
from rdflib.Graph import ConjunctiveGraph
from rdflib import plugin, URIRef, OWL, RDFS, RDF, Namespace

FOAF_NS    = Namespace('http://xmlns.com/foaf/0.1/')
WEB_ACL_NS = Namespace('http://www.w3.org/ns/auth/acl#')
WORLD      = FOAF_NS.Agent
AKAMU_WEB_ACL_NS = Namespace('https://code.google.com/p/akamu/WebACL#')

EXECUTE_MODE = AKAMU_WEB_ACL_NS.Execute

ACL_WORLD_QUERY=\
"""
PREFIX    acl: <http://www.w3.org/ns/auth/acl#>
PREFIX   foaf: <http://xmlns.com/foaf/0.1/>
ASK {
    []  acl:accessTo    %s;
        acl:mode        %s;
        acl:agentClass  foaf:Agent .
}"""

ACL_CHECK_DIRECTLY_QUERY= \
"""
PREFIX    acl: <http://www.w3.org/ns/auth/acl#>
PREFIX   foaf: <http://xmlns.com/foaf/0.1/>
ASK {
    []  a               ?class;
        foaf:name       "%s" .
    []  acl:accessTo    %s;
        acl:mode        %s;
        acl:agentClass  ?class .
}"""

ACCESS_MAP = {
    "GET"    : WEB_ACL_NS.Read,
    "PUT"    : WEB_ACL_NS.Write,
    "POST"   : EXECUTE_MODE,#WEB_ACL_NS.Append,
    "DELETE" : WEB_ACL_NS.Write,
    "PATCH"  : WEB_ACL_NS.Write,
}

class web_acl(object):
    """
    Decorator of Akara services which is expected to be used with repoze.who middleware
    and manages access to the decorated service using a (configured) AkamuGraphStore
    (specified by the first argument) comprising assertions with terms from the
    WAC vocabulary:

    http://www.w3.org/wiki/WebAccessControl/Vocabulary

    as well as assertions about users:

    []  a .. agent class ..;
        foaf:name "ikenna" .

    The classes associated with the user via rdf:type statements correspond to
    agentClasses used in statements such as:

    []  acl:accessTo    <http://example.com/service/1>;
        acl:agentClass  .. agent class .. .

    Then a service decorated this way

    @simple_service('GET', '<http://example.com/service/1>','service.1','text/plain',wsgi_wrapper=..)
    @web_acl('.. akamu graph store ..','<http://example.com/service/1>')
    def service_2():
        ..snip..

    (where .. akamu graph store .. is a graph store with the assertions above) will control
    access, ensuring that the request has been properly authenticated by repoze.who and
    that the WAC assertions indicate the user has access to the service, returning a 403 or 401
    otherwise, depending on the circumstance.

    RDF statements made using the acl:mode property are currently ignored
    """
    def __init__(self,acl_dataset,accessMap = None,simple_service=True,debug=False):
        self.simple_service         = simple_service
        self.cache                  = {}
        self.acl_dataset            = acl_dataset
        self.accessMap = ACCESS_MAP.copy()
        self.debug                  = debug
        if accessMap:
            self.accessMap.update(accessMap)

    def __call__(self, func):
        def innerHandler(*args, **kwds):
            req = Request(request.environ)
            _path = req.path[1:] if req.path[0] == '/' else req.path
            service_uri = URIRef(
                registry._current_registry._registered_services[_path].ident
            )
            user = request.environ.get('REMOTE_USER')
            if not user:
                if self.simple_service:
                    response.code = 401
                    return "Not authorized to access this resource"
                else:
                    environ, start_response = args
                    start_response("401 Unauthorized",[])
                    return "Not authorized to access this resource"
            else:
                if '_' in kwds:
                    del kwds['_']
            allowed = self.cache.get(user)
            if allowed is None:
                accessMode = self.accessMap.get(req.method)
                if accessMode is None:
                    allowed = False
                    if debug:
                        print "HTTP method not mapped, no access granted!"
                else:
                    cg =  ConjunctiveGraph(
                            ConnectToDataset(self.acl_dataset)
                    )
                    query = ACL_WORLD_QUERY%(service_uri.n3(),accessMode.n3())
                    any_user = cg.query(query).serialize(format='python')
                    if self.debug:
                        print query, any_user

                    query = ACL_CHECK_DIRECTLY_QUERY%(
                        user,
                        service_uri.n3(),
                        accessMode.n3()
                    )
                    allowed_by_group = cg.query(query).serialize(format='python')
                    if self.debug:
                        print query, allowed_by_group
                    allowed = allowed_by_group or any_user
                    self.cache[user] = allowed
            if allowed:
                if self.simple_service:
                    argInfo = inspect.getargspec(func)
                    vargs    = argInfo.varargs
                    keywords = argInfo.keywords

                    if keywords is None and argInfo.defaults:
                        keywords = argInfo.args[-len(argInfo.defaults):]
                        vargs    = argInfo.args[:-len(argInfo.defaults)]
                    if vargs and keywords:
                        return func(*args, **kwds)
                    elif vargs:
                        return func(*args)
                    elif keywords:
                        return func(**kwds)
                    else:
                        return func()
                else:
                    environ, start_response = args
                    return func(environ, start_response)
            else:
                if self.simple_service:
                    response.code = 403
                    return "The authenticated user is forbidden from accessing this resource"
                else:
                    environ, start_response = args
                    start_response("403 Forbidden",[])
                    return "The authenticated user is forbidden from accessing this resource"
        return innerHandler


__author__ = 'chimezieogbuji'

from akara.services        import service, simple_service
from akamu.protocol.sparql import graph_store_protocol, sparql_rdf_protocol
from akamu.config.dataset  import DestroyOrCreateDataset, GetGraphStoreForProtocol

SERVICE_ID = 'http://code.google.com/p/akamu/wiki/GraphStoreProtocol'

@service(SERVICE_ID, 'graph_store')
@graph_store_protocol()
def gsp_implementation(): pass

@service(SERVICE_ID, 'sparql',wsgi_wrapper=sparql_rdf_protocol('/sparql','mysqlDataset'))
def sparql_rdf_protocol_service(): pass

@simple_service('GET', SERVICE_ID, 'gsp.validator.clear','text/plain')
def validation():
    dataset,gs_url = GetGraphStoreForProtocol()
    DestroyOrCreateDataset(dataset)
    return "Reset ", dataset


# -*- encoding: utf-8 -*-
'''
Manage connection to and querying over an RDF dataset
for an Akara web application

Requires a configuration section, for example:

class dataset:
    mysqlDataset = {
        'type'         : "MySQL",
        'mysqldb'      : "[..]",
        'mysqluser'    : "[..]",
        'mysqlhost'    : "[..]",
        'mysqlpw'      : "[..]",
        'mysqlStoreId' : "[..]",
        'mysqlPort'    : "[..]"
    }

    #result XSLT directory
    result_xslt_directory = ".. /path/to/xslt/dir .."

    #Graph store to back GSP
    graph_store_name         = " .. dataset name .."
    graph_store_url          = " .. URL of GSP .."
    external_graph_store_url = ".. external URL of GSP (if different from above) .."

    #Triclops configuration
    datastore_owl = "/path/to/owl/file"
    debugQuery = True|False
    NO_BASE_RESOLUTION = True|False
    IgnoreQueryDataset = True|False
    endpointURL        = .. see: http://www.w3.org/TR/sparql11-service-description/#sd-endpoint ..

    sparqlQueryFiles = "/path/to/query/files"
    nsPrefixes       = { "..prefix.." : rdflib.Namespace  }

    sqlLiteralProps  = [ .., .., .. ]
    sqlResourceProps = [ .., .., .. ]
'''

import os, akara, time
from akara import registry
from rdflib import plugin, URIRef, OWL, RDFS, RDF, BNode
from rdflib.store import Store, NO_STORE
from rdflib.Graph import Graph, ConjunctiveGraph
from rdflib.store.SPARQL import GET,POST

OWL_PROPERTIES_QUERY=\
"""
SELECT ?literalProperty ?resourceProperty
WHERE {
    { ?literalProperty a owl:DatatypeProperty }
                    UNION
    { ?resourceProperty a ?propType
      FILTER(
        ?propType = owl:ObjectProperty ||
        ?propType = owl:TransitiveProperty ||
        ?propType = owl:SymmetricProperty ||
        ?propType = owl:InverseFunctionalProperty )  }
}"""

def GetGraphStoreForProtocol():
    configDict = akara.module_config()
    return configDict.get('graph_store_name'), configDict.get('graph_store_url')

def GetExternalGraphStoreURL():
    configDict = akara.module_config()
    return configDict.get('external_graph_store_url')

def ConfigureTriclops(datasetName,nsBindings,litProps,resProps):
    """
    Adapts akara configuration to Triclops configuration
    """
        #ontGraph,
        #ruleSet,
        #definingOntology,
        #builtinTemplateGraph,
        #defaultDerivedPreds):
    datasetConfig = akara.module_config().get(datasetName)
    connectStr = 'user=%s,password=%s,db=%s,port=%s,host=%s' % (
        datasetConfig.get('mysqluser'),
        datasetConfig.get('mysqlpw'),
        datasetConfig.get('mysqldb'),
        datasetConfig.get('mysqlPort',3306),
        datasetConfig.get('mysqlhost')
    )
    triclopsConf = {
        'result_xslt_directory' : akara.module_config().get('result_xslt_directory'),
        'store_identifier'      : datasetConfig.get('mysqlStoreId'),
        'connection'            : connectStr,
        'store'                 : datasetConfig.get('type'),
        'debugQuery'            : akara.module_config().get('debugQuery',False),
        'NO_BASE_RESOLUTION'    : akara.module_config().get('NO_BASE_RESOLUTION',False),
        'IgnoreQueryDataset'    : akara.module_config().get('IgnoreQueryDataset',False),
        'MYSQL_ORDER'           : datasetConfig.get('MYSQL_ORDER',False),
        'endpointURL'           : akara.module_config().get('endpointURL'),
    }

    proxy = None# @@TODO: Add support for proxies  global_conf.get('sparql_proxy')
    nsBindings = dict([ (k,URIRef(v)) for k,v in akara.module_config().get("nsPrefixes",{}).items()])

    dataStoreOWL = akara.module_config().get('datastore_owl')
    dataStoreOntGraph = Graph()
    if not proxy and datasetConfig.get('type') == 'MySQL':
        litProps.update(OWL.literalProperties)
        litProps.update(RDFS.literalProperties)
        resProps.update(RDFS.resourceProperties)

        litProps.update(
            map(URIRef,akara.module_config().get("sqlLiteralProps",[]))
        )
        resProps.update(
            map(URIRef,akara.module_config().get("sqlResourceProps",[]))
        )

        if dataStoreOWL:
            for dsOwl in dataStoreOWL.split(','):
                dataStoreOntGraph.parse(dsOwl)

            for litProp,resProp in dataStoreOntGraph.query(OWL_PROPERTIES_QUERY,
                initNs={u'owl':OWL_NS}):
                if litProp:
                    litProps.add(litProp)
                if resProp:
                    #Need to account for OWL Full, where datatype properties
                    #can be IFPs
                    if (resProp,
                        RDF.type,
                        OWL.DatatypeProperty) not in dataStoreOntGraph:
                        resProps.add(resProp)
        else:
            triclopsConf['datastore_owl'] = 'N/A'
        print "Registered %s owl:DatatypeProperties"%len(litProps)
        print "Registered %s owl:ObjectProperties"%len(resProps)

        if False:# @@TODO support for SPARQL RIF Core entailment global_conf.get('topDownEntailment',False):
            pass
#            from FuXi.DLP.DLNormalization import NormalFormReduction
#            from FuXi.DLP import DisjunctiveNormalForm
#            from FuXi.Horn.HornRules import HornFromDL, HornFromN3, Ruleset
#            from FuXi.Syntax.InfixOWL import *
#            from FuXi.Horn import DATALOG_SAFETY_STRICT
#            from FuXi.Rete.Magic import IdentifyDerivedPredicates
#            complementExpanded =[]
#            _ruleSet = Ruleset()
#            if global_conf.get('SkipComplementExpansion'):
#                for kvStr in global_conf.get('SkipComplementExpansion').split('|') :
#                    pref,uri=kvStr.split(':')
#                    complementExpanded.append(URIRef(nsBindings[pref]+uri))
#
#            definingOntology = global_conf.get('entailment_owl')
#            for ont in definingOntology.split(','):
#                if os.path.exists(ont):
#                    ontGraphPath = OsPathToUri(ont)
#                else:
#                    ontGraphPath = ont
#                print >>sys.stderr, "Parsing Semantic Web root Graph.. ", ontGraphPath
#                for owlImport in ontGraph.parse(ontGraphPath).objects(predicate=OWL_NS.imports):
#                    ontGraph.parse(owlImport)
#                    print >>sys.stderr, "Parsed Semantic Web Graph.. ", owlImport
#
#            for prefix,uri in nsBindings.items():
#                ontGraph.bind(prefix,uri)
#
#            builtins = global_conf.get('builtins')
#            if global_conf.get('entailment_n3'):
#                #setup rules / builtins
#                if builtins:
#                    import imp
#                    userFuncs = imp.load_source('builtins', builtins)
#                    rs = HornFromN3(global_conf.get('entailment_n3'),
#                        additionalBuiltins=userFuncs.ADDITIONAL_FILTERS)
#                else:
#                    rs = HornFromN3(global_conf.get('entailment_n3'))
#                print "Parsed %s rules from %s"%(len(rs.formulae),global_conf.get('entailment_n3'))
#                _ruleSet.formulae.extend(rs)
#
#            #Setup builtin template graph
#            builtinTemplates   = global_conf.get('builtinTemplates',False)
#            if builtinTemplates:
#                builtinTemplateGraph.parse(builtinTemplates,format='n3')
#                #setup ddl graph
#            ddlGraph = global_conf.get('ddlGraph')
#            if ddlGraph:
#                ddlGraph = Graph().parse(ddlGraph,
#                    format='n3')
#                print "Registering DDL metadata"
#                defaultDerivedPreds.extend(
#                    IdentifyDerivedPredicates(
#                        ddlGraph,
#                        ontGraph,
#                        _ruleSet))
#                #Reduce the DL expressions to a normal form
#            NormalFormReduction(ontGraph)
#            #extract rules form normalized ontology graph
#            dlp=HornFromDL(ontGraph,
#                derivedPreds=defaultDerivedPreds,
#                complSkip=complementExpansion(ontGraph))
#            _ruleSet.formulae.extend(dlp)
#            #normalize the ruleset
#            ruleSet.formulae.extend(set(DisjunctiveNormalForm(_ruleSet,safety=DATALOG_SAFETY_STRICT)))
    return triclopsConf

def ReplaceGraph(datasetOrName,
                 graphUri,
                 srcStream,
                 format='xml',
                 storeName=True,
                 baseUri=None,
                 smartDiff=False,
                 debug=False):
    #TODO: do a lazy replace (only the diff - ala 4Suite repository)
    store = ConnectToDataset(datasetOrName) if storeName else datasetOrName
    g = Graph(store, graphUri)
    if smartDiff:
        def hasBNodes(triple):
            return filter(lambda term:isinstance(term,BNode),triple)
        new_graph = Graph().parse(srcStream,publicID=baseUri)
        stmsToAdd = [ s for s in new_graph
                        if s not in g or hasBNodes(s) ]
        stmsToDel = [ s for s in g
                        if s not in new_graph or hasBNodes(s) ]
        for s in stmsToDel:
            g.remove(s)
        for s in stmsToAdd:
            g.add(s)
        if debug:
            print "Removed %s triples and added %s from/to %s"%(
                len(stmsToDel),
                len(stmsToAdd),
                graphUri
            )
    else:
        g.remove((None, None, None))
        g.parse(srcStream,publicID=baseUri)
    store.commit()

def ClearGraph(datasetOrName,graphUri,storeName=True):
    #TODO: do a lazy replace (only the diff - ala 4Suite repository)
    store = ConnectToDataset(datasetOrName) if storeName else datasetOrName
    g = Graph(store, graphUri)
    g.remove((None, None, None))
    store.commit()

def DestroyOrCreateDataset(datasetName):
    """
    Initialize dataset (if exists) or create it if it doesn't
    """
    datasetConfig = akara.module_config().get(datasetName)
    assert datasetConfig is not None, datasetName
    if datasetConfig['type'] == 'MySQL':
        configStr = 'user=%s,password=%s,db=%s,port=%s,host=%s' % (
            datasetConfig.get('mysqluser'),
            datasetConfig.get('mysqlpw'),
            datasetConfig.get('mysqldb'),
            datasetConfig.get('mysqlPort',3306),
            datasetConfig.get('mysqlhost')
            )
        store = plugin.get('MySQL', Store)(datasetConfig.get('mysqlStoreId'))
        rt = store.open(configStr,create=False)
        if rt == NO_STORE:
            store.open(configStr,create=True)
        else:
            store.destroy(configStr)
            store.open(configStr,create=True)
        return store
    else:
        raise NotImplementedError("Only dataset supported by Akamu is MySQL")


def ConnectToDataset(datasetName):
    """
    Return rdflib store corresponding to the named dataset, whose connection
    parameters are specified in the configuration file
    """
    datasetConfig = akara.module_config().get(datasetName)
    assert datasetConfig is not None
    if datasetConfig['type'] == 'MySQL':
        configStr = 'user=%s,password=%s,db=%s,port=%s,host=%s' % (
            datasetConfig.get('mysqluser'),
            datasetConfig.get('mysqlpw'),
            datasetConfig.get('mysqldb'),
            datasetConfig.get('mysqlPort',3306),
            datasetConfig.get('mysqlhost')
        )
        store = plugin.get('MySQL', Store)(datasetConfig.get('mysqlStoreId'))
        store.open(configStr, create=False)
        store.literal_properties.update(
            map(URIRef,akara.module_config().get("sqlLiteralProps",[]))
        )
        store.resource_properties.update(
            map(URIRef,akara.module_config().get("sqlResourceProps",[]))
        )
        return store
    elif datasetConfig['type'] == 'SPARQLService':
        if 'endpoint' not in datasetConfig:
            raise SyntaxError('Missing "endpoint" directive')
        sparql_store = plugin.get('SPARQL', Store)(datasetConfig.get('endpoint'))
        for k,v in datasetConfig.get('extraQueryParams',{}).items():
            sparql_store._querytext.append((k,v))
        sparql_store.method = POST if datasetConfig.get(
            'method','GET').lower() == 'post' else GET
        return sparql_store
    else:
        raise NotImplementedError("Only dataset supported by Akamu is MySQL")

def Ask(queryFile,datasetName,graphUri=None,params=None,debug=False):
    """
    Same as Query but where query is ASK (returns boolean)
    """
    store = ConnectToDataset(datasetName)
    g     = ConjunctiveGraph(store) if graphUri is None else Graph(store,graphUri)
    qFile = os.path.join(akara.module_config().get("sparqlQueryFiles"),queryFile)
    query = open(qFile).read()
    query = query if params is None else query % params
    if debug:
        print query
    initNs = dict([ (k,URIRef(v)) for k,v in akara.module_config().get("nsPrefixes",{}).items()])
    if debug:
        then = time.time()
        rt = g.query(query,initNs=initNs,DEBUG=debug).serialize(format='python')
        print "Query time", time.time() - then
    else:
        rt = g.query(query,initNs=initNs,DEBUG=debug).serialize(format='python')
    return rt

def Query(queryFile,datasetName,graphUri=None,params=None,debug=False):
    """
    Evaluate a query (stored in a SPARQL file in the location indicated in the
    configuration) against the given dataset (and optional named graph within it)
    using the optional parameters given
    """
    store = ConnectToDataset(datasetName)
    g     = ConjunctiveGraph(store) if graphUri is None else Graph(store,graphUri)
    qFile = os.path.join(akara.module_config().get("sparqlQueryFiles"),queryFile)
    query = open(qFile).read()
    query = query if params is None else query % params
    if debug:
        print query
    initNs = dict([ (k,URIRef(v)) for k,v in akara.module_config().get("nsPrefixes",{}).items()])
    for rt in g.query(
        query,
        initNs=initNs,
        DEBUG=debug):
        yield rt

def GetParameterizedQuery(queryFile,params=None):
    qFile = os.path.join(akara.module_config().get("sparqlQueryFiles"),queryFile)
    query = open(qFile).read()
    return query if params is None else query % params


__author__ = 'chimezieogbuji'

import unittest, amara, os, urllib2
from cStringIO import StringIO

from shutil import rmtree

from server_support import server

from rdflib.Graph import Graph, ConjunctiveGraph
from rdflib import URIRef, plugin, RDF, RDFS, BNode, Namespace, Literal
from rdflib.util import first

from akamu.diglot import Manager, Resource
from akamu.config.dataset import DestroyOrCreateDataset
from akamu.xslt import TransformWithAkamuExtensions
from amara import bindery

from IsomorphicTestableGraph import IsomorphicTestableGraph

from amara.namespaces import *
from amara import tree
from amara.writers.struct import structwriter, E, NS, ROOT
from amara.lib import U

VCARD = Namespace('http://www.w3.org/2006/vcard/ns#')
FOAF  = Namespace('http://xmlns.com/foaf/0.1/')

TEST_ROOT_DIR_PATH  = '/tmp/diglotDir'
TEST_DIR_PATH       = '/tmp/diglotDir/patients'
TEST_XSLT_PATH      = '/tmp/diglotDir/xslt'

def TestGraphUriFn(path,fName):
    return 'http://example.com%s'%path.split('.')[0]

class TestDiglotCapabilities(unittest.TestCase):
    def setUp(self):
        os.makedirs(TEST_DIR_PATH)
        os.makedirs(TEST_XSLT_PATH)
        f=open(os.path.join(TEST_XSLT_PATH,'grddl.xslt'),'w')
        f.write(open('test/foaf.xslt').read())
        f.close()

        self.store = Graph().store
        self.mgr = Manager(TEST_ROOT_DIR_PATH,
                           self.store,
                           graphUriFn=TestGraphUriFn,
                           transforms4Dir={
                               u'/patients' : u'/xslt/grddl.xslt'
                           })
        src    = StringIO()
        w = structwriter(indent=u"yes", stream=src)
        w.feed(
            ROOT(
                E(u'Patient',{ u'name' : u'Chimezie Ogbuji', u'gender' : u'Male'})
            )
        )
        self.ptDoc = src.getvalue()
        self.ptUri   = URIRef('http://example.com/patients/ptRec01')
        self.server = server()

    def test_create_and_mirrored(self):

        path = '/patients/ptRec01.xml'

        self.mgr.createResource(path,self.ptDoc)

        ptGraph = Graph(self.store,identifier=self.ptUri)

        msg = "No faithful rendition of %s in %s!"%(
            '/patients/ptRec01.xml',
            self.ptUri
        )
        self._test_via_sparql(ptGraph, msg, "Chimezie Ogbuji")

    def test_create_and_update(self):

        path = '/patients/ptRec01.xml'

        res = self.mgr.createResource(path,self.ptDoc)

        ptGraph = Graph(self.store,identifier=self.ptUri)

        msg = "No faithful rendition of %s in %s!"%(
            '/patients/ptRec01.xml',
            self.ptUri
        )
        self._test_via_sparql(ptGraph, msg, "Chimezie Ogbuji")

        src    = StringIO()
        w = structwriter(indent=u"yes", stream=src)
        w.feed(
            ROOT(
                E(u'Patient',{ u'name' : u'Uche Ogbuji', u'gender' : u'Male'})
            )
        )
        res.update(src.getvalue())

        ptGraph = Graph(self.store,identifier=self.ptUri)
        self._test_via_sparql(ptGraph, "Name wasn't updated", "Uche Ogbuji")

        rt = list(ptGraph.query(
            'SELECT ?NAME { [] a foaf:Person; foaf:businessCard [ v:fn ?NAME ] } ',
            initNs={
                u'foaf' : FOAF,
                u'v'    : VCARD }
        ))
        self.failUnless(len(rt)==1,"More than one name in graph (should only be Uche)")
        self.failUnless(first(rt) == Literal('Uche Ogbuji'),"Old name still in faithful rendition")

    def test_get_resource(self):

        path = '/patients/ptRec01.xml'

        self.mgr.createResource(path,self.ptDoc)

        doc = amara.parse(self.mgr.getResource(path).getContent())

        self.failUnless(doc.xml_select('/Patient') and
                        doc.xml_select('/Patient[@name = "Chimezie Ogbuji"]'))
        self.failUnless(doc.xml_select('/Patient'))

    def test_delete_resource(self):
        path = '/patients/ptRec01.xml'

        self.mgr.createResource(path,self.ptDoc)

        self.mgr.deleteResource('/patients/ptRec01.xml')
        ptGraph = Graph(self.store,identifier=self.ptUri)

        self.failUnless(not len(ptGraph),"Faithful rendition should be erased with removal of resource")

    def _test_via_sparql(self, graph, msg, name):
        query = 'ASK { [] a foaf:Person; foaf:businessCard [ v:fn "%s" ] } '%(
            name
        )
        queryAns = graph.query(
            query,
            initNs={
                u'foaf': FOAF,
                u'v'   : VCARD}
        )
        self.failUnless(queryAns.askAnswer[0], msg)

    def test_grddl_graph_store(self):
        from GRDDLAmara import WebMemo, GRDDLAgent

        path = '/patients/ptRec01.xml'

        req = urllib2.Request(
            self.server[:-1]+'/diglot'+path,
            headers={ "Content-Type" : "application/xml" }
        )
        req.get_method = lambda: 'PUT'
        urllib2.urlopen(req,data=self.ptDoc)

#        self.mgr.createResource(path,self.ptDoc)

#        ptGraph = Graph(self.store,identifier=self.ptUri)
#
#        msg = "No faithful rendition of %s in %s!"%(
#            '/patients/ptRec01.xml',
#            self.ptUri
#        )
#        self._test_via_sparql(ptGraph, msg, "Chimezie Ogbuji")

        fullPath = "diglot%s"%path

        grddlResult = IsomorphicTestableGraph()
        GRDDLAgent(
            self.server + fullPath,
            grddlResult,
            WebMemo(),
            useXInclude=True,
            DEBUG = True
        )

        msg = "GET to %s didn't return GRDDL source document"%(
            self.server + fullPath
        )
        self._test_via_sparql(grddlResult, msg, "Chimezie Ogbuji")

        personUri = first(grddlResult.subjects(RDF.type,FOAF.Person))

        #POSTing XML to transform returns the GRDDL result
        req = urllib2.Request(
            self.server + "diglot/xslt/grddl.xslt",
            headers={ "Content-Type" : "application/xml" }
        )
        response = urllib2.urlopen(req,data=self.ptDoc)
        rt=response.read()
        responseGraph = IsomorphicTestableGraph().parse(StringIO(rt),publicID=personUri)

        self.failUnless(grddlResult == responseGraph,
            "Incorrect GRDDL result returned by POST to transform"
        )

        #POSTing RDF to transform returns the corresponding XML source
        #(using reverse transform)
        req = urllib2.Request(
            self.server + "diglot/xslt/grddl.xslt?base=%s"%urllib2.quote(personUri),
            headers={ "Content-Type" : "application/rdf+xml" }
        )
        response = urllib2.urlopen(req,data=grddlResult.serialize(format='pretty-xml'))
        doc = bindery.parse(response.read())
        self.failUnless(
            doc.xml_select('/Patient[@name = "Chimezie Ogbuji"]'),
            "Document doesn't match GRDDL source: %s"%rt
        )

        #POSTing XML to directory will add it to the directory using a generated
        #file name, which is returned in Location header
        src    = StringIO()
        w = structwriter(indent=u"yes", stream=src)
        w.feed(
            ROOT(
                E(u'Patient',{ u'name' : u'Uche Ogbuji'})
            )
        )
        xmlSrc = src.getvalue()
        req = urllib2.Request(
            self.server + "diglot/patients",
            headers={ "Content-Type" : "application/xml" }
        )
        response = urllib2.urlopen(req,data=xmlSrc)
        self.failUnless('Location' in response.info(),"No Location header returned")
        self.failUnless(response.getcode() == 201,"Didn't respond with 201 Created")
        location = response.info()['Location']

        grddlResult = Graph()
        GRDDLAgent(
            self.server[:-1]+location,
            grddlResult,
            WebMemo(),
            useXInclude=True,
            DEBUG = True
        )

        msg = "GET to %s didn't return GRDDL source document" % location
        self._test_via_sparql(grddlResult, msg, "Uche Ogbuji")

        #POSTing RDF to directory is same as GSP (using reverse mapping)
        req = urllib2.Request(
            self.server + "diglot/patients",
            headers={ "Content-Type" : "application/rdf+xml" }
        )
        response = urllib2.urlopen(req,data=grddlResult.serialize(format='pretty-xml'))
        self.failUnless('Location' in response.info(),"No Location header returned")
        self.failUnless(response.getcode() == 201,"Didn't respond with 201 Created")
        location = response.info()['Location']

        grddlResult = Graph()
        GRDDLAgent(
            self.server[:-1]+location,
            grddlResult,
            WebMemo(),
            useXInclude=True,
            DEBUG = True
        )

        msg = "GET to %s didn't return GRDDL source document" % location
        self._test_via_sparql(grddlResult, msg, "Uche Ogbuji")

        #XUpdate to change name
        xupdate = """<?xml version="1.0"?>
<xupdate:modifications
  version="1.0"
  xmlns:xupdate="http://www.xmldb.org/xupdate">
  <xupdate:update select="/Patient/@name">Chimezie Ogbuji</xupdate:update>
</xupdate:modifications>"""
        req = urllib2.Request(
            self.server[:-1]+location,
            headers={ "Content-Type" : "application/xml" }
        )
        req.get_method = lambda: 'PATCH'
        urllib2.urlopen(req,data=xupdate)

        req = urllib2.Request(self.server[:-1]+location)
        doc = bindery.parse(urllib2.urlopen(req).read())
        self.failUnless(
            doc.xml_select('/Patient[@name = "Chimezie Ogbuji"]'),
            "Document doesn't match GRDDL source: %s"%rt
        )


        #XSLT to change name
        xsltSrc = """<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet
        xmlns:xsl   = "http://www.w3.org/1999/XSL/Transform"
        version     = "1.0">
    <xsl:template match="/">
        <xsl:apply-templates  />
    </xsl:template>
    <xsl:template match="@name">
        <xsl:attribute name="name" >Uche Ogbuji</xsl:attribute>
    </xsl:template>
    <xsl:template match="@*|node()">
        <xsl:copy>
          <xsl:apply-templates select="@*|node()"/>
        </xsl:copy>
    </xsl:template>
</xsl:stylesheet> """
        req = urllib2.Request(
            self.server[:-1]+location,
            headers={ "Content-Type" : "application/xslt+xml" }
        )
        req.get_method = lambda: 'PATCH'
        urllib2.urlopen(req,data=xsltSrc)

        req = urllib2.Request(self.server[:-1]+location)
        doc = bindery.parse(urllib2.urlopen(req).read())
        self.failUnless(
            doc.xml_select('/Patient[@name = "Uche Ogbuji"]'),
            "Document doesn't match GRDDL source: %s"%rt
        )


        req = urllib2.Request(self.server[:-1]+location)
        req.get_method = lambda: 'DELETE'
        resp = urllib2.urlopen(req)
        self.failUnless(resp.getcode() == 204,"Wrong response code: %s"%resp.getcode())

        notFound = False
        try:
            urllib2.urlopen(self.server[:-1]+location)
        except urllib2.URLError, e:
            if e.code == 404: notFound = True

        self.failUnless(notFound,"%s should have been removed by DELETE"%(self.server[:-1]+location))

        src    = StringIO()
        w = structwriter(indent=u"yes", stream=src)
        w.feed(
            ROOT(
                E(u'Patient',{ u'name' : u'Ejike Ogbuji'})
            )
        )
        xmlSrc = src.getvalue()

        req = urllib2.Request(
            self.server[:-1]+'/diglot'+path,
            headers={ "Content-Type" : "application/xml" }
        )
        req.get_method = lambda: 'PUT'
        urllib2.urlopen(req,data=xmlSrc)

        grddlResult = Graph()
        GRDDLAgent(
            self.server[:-1]+'/diglot'+path,
            grddlResult,
            WebMemo(),
            useXInclude=True,
            DEBUG = True
        )
        msg = "GET to resource created by PUT (/patients/ptRec01.xml) didn't return GRDDL source document"
        self._test_via_sparql(grddlResult, msg, "Ejike Ogbuji")

    def tearDown(self):
        rmtree(TEST_ROOT_DIR_PATH)

class TestAkamuExtensions(unittest.TestCase):
    def setUp(self):
        os.makedirs(TEST_DIR_PATH)
        os.makedirs(TEST_XSLT_PATH)
        f=open(os.path.join(TEST_XSLT_PATH,'grddl.xslt'),'w')
        f.write(open('test/foaf.xslt').read())
        f.close()

        self.server = server()

    def test_diglot_extensions_basic(self):
        req = urllib2.Request(self.server + "diglot_extensions_basic_test?rootPath=%s"%TEST_ROOT_DIR_PATH)
        response = urllib2.urlopen(req).read()
        self.failUnless(response == 'Success')

    def tearDown(self):
        rmtree(TEST_ROOT_DIR_PATH)

if __name__ == '__main__':
#    suite = unittest.TestLoader().loadTestsFromTestCase(TestAkamuExtensions)
#    unittest.TextTestRunner(verbosity=3).run(suite)
    suite = unittest.TestLoader().loadTestsFromTestCase(TestDiglotCapabilities)
    unittest.TextTestRunner(verbosity=3).run(suite)


import json, os, cgi, urllib, re
from pprint import pprint
from amara.writers.struct import structwriter, E, ROOT
from cStringIO import StringIO
from akara import module_config as config, request
from akara.services  import service, simple_service
from akamu.config.dataset import Query
from akamu.xslt import xslt_rest, NOOPXML

SERVICE_ID_PREFIX = u'http://example.com/service/'
HTML_IMT   = 'text/html'
JSON_IMT   = 'application/json'

@simple_service(
    'GET' ,
    SERVICE_ID_PREFIX + 'music_brainz_browse_artist',
    'browse.artist',
    HTML_IMT)
@xslt_rest(
    os.path.join(
        config().get('working_directory'),
        'test/musicbrainz/music_brainz_browse_artist.xslt'),
)
def browse_artist(artist_uri):
    names           = set()
    tracks          = set()
    for artistName, track, trackLabel in Query(
            'musicbrainz-artist-info.rq',
            'musicbrainz',
            params=(artist_uri,artist_uri)):
        names.add(artistName)
        tracks.add(trackLabel)
    src = StringIO()
    w = structwriter(indent=u"yes", stream=src)
    w.feed(ROOT(
        E(u'Music',{u'artist': ' / '.join(names)},
          E(u'Tracks',
            (
                E(u'Track',{u'name':trackLabel}) for trackLabel in tracks
            )
          ),
        )
    ))
    return src.getvalue()

@simple_service(
    'GET' ,
    SERVICE_ID_PREFIX + 'music_brainz_form',
    'search.form',
    HTML_IMT)
@xslt_rest(
    os.path.join(
        config().get('working_directory'),
        'test/musicbrainz/music_brainz_search.xslt'),
    source=NOOPXML
)
def search_form(): pass

@simple_service('GET', SERVICE_ID_PREFIX + 'categories', 'categories',JSON_IMT)
def categories(term=None):
    return json.dumps([
        { 'label' : label,
          'value' : label }
            for tag,label in Query('musicbrainz-categories.rq',
                                   'musicbrainz',
                                   params=(term,))
    ])

@simple_service(
    'POST' ,
    SERVICE_ID_PREFIX + 'music_brainz_search',
    'search',
    HTML_IMT)
@xslt_rest(
    os.path.join(
        config().get('working_directory'),
        'test/musicbrainz/music_brainz_search.xslt'),
)
def search(body,ctype):
    form = cgi.FieldStorage(
        fp=StringIO(body),
        environ=request.environ
    )
    category = form.getvalue('category')
    artist   = form.getvalue('artistSearch')
    pattern = re.compile(artist)
    src = StringIO()
    w = structwriter(indent=u"yes", stream=src)

    if category is not None and category.strip():
        queryFile   = 'musicbrainz-artists.rq'
        params      = (category,)
    else:
        queryFile   = 'musicbrainz-artists-any-category.rq'
        params      = None

    w.feed(ROOT(
        E(u'Root',
          E(u'Artists',
            (
                E(u'Artist',
                  {u'name' : name,
                   u'url'  : urllib.quote(artist_uri)})
                for artist_uri,
                    name in Query(queryFile,'musicbrainz',params=params)
                if pattern.match(name)
            )
          )
        )
    ))
    return src.getvalue()


__author__ = 'chimezieogbuji'

import unittest, urllib, urllib2
from cStringIO import StringIO
from server_support import server
from rdflib.Graph import Graph
from rdflib import Namespace
from akamu.config.dataset import DestroyOrCreateDataset
from akamu.xslt import TransformWithAkamuExtensions

VCARD = Namespace('http://www.w3.org/2006/vcard/ns#')
FOAF  = Namespace('http://xmlns.com/foaf/0.1/')

NS_MAPPING = {
    u'foaf' : FOAF,
    u'v'    : VCARD
}

class TestRestTransform(unittest.TestCase):
    def setUp(self):
        self.server = server()

    def test_invoke_get(self):
        req = urllib2.Request(self.server + "xslt_rest_get?name=Uche+Ogbuji")
        response = urllib2.urlopen(req)
        g = Graph().parse(response)

        queryAns = g.query(
            'ASK { [] a foaf:Person; foaf:businessCard [ v:fn "Uche Ogbuji" ] } ',
            initNs=NS_MAPPING
        )
        self.failUnless(queryAns.askAnswer[0])

        req = urllib2.Request(self.server + "xslt_rest_get?name=Uche+Ogbuji")
        response = urllib2.urlopen(req)
        g = Graph().parse(response)

        queryAns = g.query(
            'ASK { [] a foaf:Person; foaf:businessCard [ v:fn "Chimezie Ogbuji" ] } ',
            initNs=NS_MAPPING
        )
        self.failUnless(queryAns.askAnswer[0])

    def _test_invoke_post(self):
        from httplib2 import Http
        h = Http()
        resp, content = h.request(
                self.server + "xslt_rest_post",
                "POST",
                urllib.urlencode({'name': 'Uche Ogbuji'})
        )
        g = Graph().parse(StringIO(content))

        queryAns = g.query(
            'ASK { [] a foaf:Person; foaf:businessCard [ v:fn "Uche Ogbuji" ] } ',
            initNs=NS_MAPPING
        )
        self.failUnless(queryAns.askAnswer[0])

        h = Http()
        resp, content = h.request(
            self.server + "xslt_rest_post",
            "POST",
            urllib.urlencode({})
        )

        g = Graph().parse(StringIO(content))

        queryAns = g.query(
            'ASK { [] a foaf:Person; foaf:businessCard [ v:fn "Chimezie Ogbuji" ] } ',
            initNs=NS_MAPPING
        )
        self.failUnless(queryAns.askAnswer[0])

class TestAkamuExtensions(unittest.TestCase):
    def setUp(self):
        os.makedirs(TEST_DIR_PATH)
        os.makedirs(TEST_XSLT_PATH)
        f=open(os.path.join(TEST_XSLT_PATH,'grddl.xslt'),'w')
        f.write(open('test/foaf.xslt').read())
        f.close()

        self.server = server()

    def test_diglot_extensions_basic(self):
        TransformWithAkamuExtensions(src,xform,baseUri,params,manager)

if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestRestTransform)
    unittest.TextTestRunner(verbosity=5).run(suite)


#download ez_setup.py from http://peak.telecommunity.com/dist/ez_setup.py
#import ez_setup
#ez_setup.use_setuptools()
from setuptools  import setup
setup(name="Akamu",
      version="0.7",
      description="An Akara module for managing the use of an RDF dataset, a "+
                  "XML/RDF filesystem, and XSLT extension functions within a "+
                  "web application",
      package_dir = {
        'akamu': 'lib',
      },
      packages=[
        "akamu",
        "akamu.protocol",
        "akamu.wheezy",
        "akamu.security",
        "akamu.xslt",
        "akamu.demo",
        "akamu.util",
        "akamu.diglot",
        "akamu.config",
      ],
      install_requires = [
        'python-memcached',
        'akara',
        'amara',
        'wheezy.http==0.1.297',
        'wheezy.core==0.1.104',
        'wheezy.caching==0.1.90',
        'webob',
        'httpagentparser',
#        'httplib2'
      ],
      zip_safe=False
)


__author__ = 'chimezieogbuji'

import unittest, urllib, urllib2, amara
import httplib2
from cStringIO import StringIO
from server_support import server
from rdflib.Graph import Graph
from rdflib import Namespace

VCARD = Namespace('http://www.w3.org/2006/vcard/ns#')
FOAF  = Namespace('http://xmlns.com/foaf/0.1/')

NS_MAPPING = {
    u'foaf' : FOAF,
    u'v'    : VCARD
}

class DefaultErrorHandler(urllib2.HTTPDefaultErrorHandler):
    def http_error_default(self, req, fp, code, msg, headers):
        result = urllib2.HTTPError(
            req.get_full_url(), code, msg, headers, fp)
        result.status = code
        return result

class NotModifiedHandler(urllib2.BaseHandler):

    def http_error_304(self, req, fp, code, message, headers):
        addinfourl = urllib2.addinfourl(fp, headers, req.get_full_url())
        addinfourl.code = code
        return addinfourl

class TestWheezyWrapper(unittest.TestCase):
    def setUp(self):
        self.server = server()

    def test_invoke_get(self):
        addr = self.server + "wheezy.test"
#        print addr
#        h = httplib2.Http(".cache")
#        md, content = h.request(addr)
#        md, content = h.request(addr,headers={'cache-control':'no-cache'})
#        print md['etag'], md.status, md
#        etag = md['etag']
#        code = md.status

        response = urllib2.urlopen(addr)
        code = response.getcode()
        self.failUnless(code == 200)
        headers = response.info()
        etag = headers.getheader("ETag")
        self.failUnless(etag == '12345')
        doc = amara.parse(response)
        self.failUnless(
            doc.xml_select(
                '/xhtml:html/xhtml:body/xhtml:p',
                prefixes={
                    u'xhtml':u'http://www.w3.org/1999/xhtml' }
            )
        )

        request = urllib2.Request(addr)
        opener = urllib2.build_opener(DefaultErrorHandler())
        request.add_header('If-None-Match',etag)
#        print request.headers
        seconddatastream = opener.open(request)
        code = seconddatastream.getcode()
#        print seconddatastream.headers
        self.failUnless(code == 304,"%s is not 304"%code)

#        response = urllib2.urlopen(self.server + "wheezy.test.clear_cache")
#        code = response.getcode()
#        self.failUnless(code == 200)
#
#        request = urllib2.Request(addr)
#        opener = urllib2.build_opener(DefaultErrorHandler())
#        request.add_header('If-None-Match',etag)
#        print request.headers
#        seconddatastream = opener.open(request)
#        code = seconddatastream.getcode()
#        print seconddatastream.headers
#        self.failUnless(code == 200,"%s is not 200"%code)

if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestWheezyWrapper)
    unittest.TextTestRunner(verbosity=5).run(suite)


import unittest, os, urllib, urllib2, re, httplib2, shutil, base64
from rdflib.store import Store, NO_STORE
from rdflib.Graph import Graph, ConjunctiveGraph
from rdflib import plugin, BNode, Namespace, Literal, OWL
from server_support import server
from akamu.diglot import Manager, Resource

TEST_ROOT_DIR_PATH  = '/tmp/diglotDir'
TEST_DIR_PATH       = '/tmp/diglotDir/acl'
TEST_XSLT_PATH      = '/tmp/diglotDir/xslt'

def TestGraphUriFn(path,fName):
    return 'http://example.com%s'%path.split('.')[0]

class TestWebACLCapabilities(unittest.TestCase):
    def setUp(self):
        mysqluser,mysqlpw,mysqlhost,mysqldb = os.environ.get(
            "MYSQL_CREDENTIALS",
            ",,,"
        ).split(',')
        configStr = 'user=%s,password=%s,db=%s,host=localhost'%(
            mysqluser,
            mysqlpw,
            mysqldb
        )
        store = plugin.get('MySQL', Store)('test')
        rt = store.open(configStr,create=False)
        if rt == NO_STORE:
            store.open(configStr,create=True)
        else:
            store.destroy(configStr)
            store.open(configStr,create=True)
        store.open(configStr,create=False)

        os.makedirs(TEST_DIR_PATH)
        os.makedirs(TEST_XSLT_PATH)
        f=open(os.path.join(TEST_XSLT_PATH,'acl_grddl.xslt'),'w')
        f.write(open('test/acl_grddl.xslt').read())
        f.close()

        self.store = store
        self.mgr = Manager(TEST_ROOT_DIR_PATH,
                           self.store,
                           graphUriFn=TestGraphUriFn,
                           transforms4Dir={
                               u'/acl' : u'/xslt/acl_grddl.xslt'
                           })
        path = '/acl/1.xml'
        self.mgr.createResource(path,open('test/test_user.xml').read())
        self.server = server()

    def tearDown(self):
        shutil.rmtree(TEST_DIR_PATH)
        shutil.rmtree(TEST_XSLT_PATH)

    def test_basic_unauthorized(self):
        url_pattern = re.compile("http://localhost:[0-9]+/login\?came_from")
        req = urllib2.Request(self.server + "service.1")

        handler = SmartRedirectHandler()
        opener = urllib2.build_opener(handler)
        self.failUnlessRaises(urllib2.HTTPError,opener.open,req)
        self.failUnless('Location' in handler.headers,"No Location header returned")
        self.failUnless(url_pattern.match(handler.headers['Location']),
                        "Didn't redirect to /login")

    def test_any_authorized(self):
        h = httplib2.Http()
        h.add_credentials('admin', 'admin',"repoze.who")
        resp, content = h.request(
            self.server + "service.2",
            headers = { "Authorization" :
                            "Basic {0}".format(
                                base64.b64encode("{0}:{1}".format('admin', 'admin')))})
        self.failUnlessEqual(resp.status,200)
        self.failUnlessEqual(content,"Success")

    def test_group_authorized(self):
        h = httplib2.Http()
        h.add_credentials('admin', 'admin',"repoze.who")
        resp, content = h.request(
            self.server + "service.1",
            headers = { "Authorization" :
                            "Basic {0}".format(
                                base64.b64encode("{0}:{1}".format('admin', 'admin')))})
        self.failUnlessEqual(resp.status,200)
        self.failUnlessEqual(content,"Success")

        h = httplib2.Http()
        h.add_credentials('admin', 'admin',"repoze.who")
        resp, content = h.request(
            self.server + "service.3",
            headers = { "Authorization" :
                            "Basic {0}".format(
                                base64.b64encode("{0}:{1}".format('admin', 'admin')))})
        self.failUnlessEqual(resp.status,403)

    def test_no_write(self):
        h = httplib2.Http()
        h.add_credentials('admin', 'admin',"repoze.who")
        resp, content = h.request(
            self.server + "service.1","POST", body="This is text",
            headers = { "Authorization" :
                            "Basic {0}".format(
                                base64.b64encode("{0}:{1}".format('admin', 'admin'))) ,
                        'content-type':'text/plain'}
        )
        self.failUnlessEqual(resp.status,403)

    def test_append_no_write(self):
        h = httplib2.Http()
        h.add_credentials('admin', 'admin',"repoze.who")
        resp, content = h.request(
            self.server + "service.2","PUT", body="This is text",
            headers = { "Authorization" :
                            "Basic {0}".format(
                                base64.b64encode("{0}:{1}".format('admin', 'admin'))),
                        'content-type':'text/plain'}
        )
        self.failUnlessEqual(resp.status,403)

        h = httplib2.Http()
        h.add_credentials('admin', 'admin',"repoze.who")
        resp, content = h.request(
            self.server + "service.2","POST", body="This is text",
            headers = { "Authorization" :
                            "Basic {0}".format(
                                base64.b64encode("{0}:{1}".format('admin', 'admin'))),
                        'content-type':'text/plain'}
        )
        self.failUnlessEqual(resp.status,200)
        self.failUnlessEqual(content,"Success")

class SmartRedirectHandler(urllib2.HTTPRedirectHandler):
    def http_error_301(self, req, fp, code, msg, headers):
        result = urllib2.HTTPRedirectHandler.http_error_301(
            self, req, fp, code, msg, headers)
        result.status = code
        return result

    def http_error_302(self, req, fp, code, msg, headers):
        self.headers = headers
        result = urllib2.HTTPRedirectHandler.http_error_302(
            self, req, fp, code, msg, headers)
        result.status = code
        return result

if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestWebACLCapabilities)
    unittest.TextTestRunner(verbosity=3).run(suite)


# -*- coding: utf-8 -*-
import httplib2, os,cgi,urllib2,datetime
from akara                          import request, logger, module_config as config
from rdflib                         import URIRef
from rdflib.Graph                   import Graph
from rdflib_tools.GraphIsomorphism  import IsomorphicTestableGraph
from amara.lib                      import iri
from amara.lib.util                 import *
from amara.xslt                     import transform
from amara.writers.struct           import structwriter, E, NS, ROOT, RAW
from akara.services                 import simple_service, service
from akara                          import response
from akamu.xslt                     import xslt_rest, NOOPXML
from akamu.config.dataset           import GetParameterizedQuery
from akamu.diglot                   import layercake_mimetypes, XML_MT, layercake_parse_mimetypes
from cStringIO                      import StringIO
from urlparse                       import urlparse
from webob                          import Request

XHTML_IMT  = 'application/xhtml+xml'
HTML_IMT   = 'text/html'
XML_IMT    = 'application/xml'
SERVICE_ID = 'http://code.google.com/p/akamu/wiki/GraphStoreProtocol'
TEST_NS    = 'http://www.w3.org/2009/sparql/docs/tests/data-sparql11/http-rdf-update/tests.html'

@simple_service('GET', SERVICE_ID, 'gsp.validator.form',HTML_IMT+';charset=utf-8')
@xslt_rest(
    os.path.join(
        config().get('demo_path'),
        'gsp_validator.xslt'))
def validator_form(message=None):
    if message:
        return NOOPXML, { u'message':message }
    else:
        return NOOPXML

def post_multipart(host, selector, files):
    """
    from http://code.activestate.com/recipes/146306/

    Post fields and files to an http host as multipart/form-data.
    fields is a sequence of (name, value) elements for regular form fields.
    files is a sequence of (name, filename, value) elements for data to be uploaded as files
    Return the server's response page.
    """
    import httplib
    content_type, body = encode_multipart_formdata(files)
    h = httplib.HTTPSConnection(host)
    header = {
        'Content-Type'  : content_type,
        'Content-Length': len(body)
    }
    h.request('POST', selector, body, header)
    res = h.getresponse()
    return res.status, res.reason, res.read()

def encode_multipart_formdata(files):
    """
    from http://code.activestate.com/recipes/146306/

    fields is a sequence of (name, value) elements for regular form fields.
    files is a sequence of (name, filename, value) elements for data to be uploaded as files
    Return (content_type, body) ready for httplib.HTTP instance
    """
    BOUNDARY1 = 'ThIs_Is_tHe_outer_bouNdaRY_$'
    BOUNDARY2 = 'ThIs_Is_tHe_inner_bouNdaRY_$'
    CRLF = '\r\n'
    L = []

    L.append('--' + BOUNDARY1)
    L.append('Content-Disposition: form-data; name="graphs"')
    L.append('Content-Type: multipart/mixed; boundary=%s'%BOUNDARY2)
    L.append('')

    for (filename, mtype, value) in files:
        L.append('--' + BOUNDARY2)
        L.append('Content-Disposition: file; filename="%s"' % (filename,))
        L.append('Content-Type: %s' % mtype)
        L.append('')
        L.append(value)
    L.append('--' + BOUNDARY2)
    L.append('--' + BOUNDARY1)
#    L.append('')
    body = CRLF.join(L)
    content_type = 'multipart/form-data; boundary=%s' % BOUNDARY1
    return content_type, body

TESTS = [
    "GET of replacement for empty graph",
    "PUT - replace empty graph",
    "PUT - Initial state",
    "GET of PUT - Initial state",
    "PUT - graph already in store",
    "GET of PUT - graph already in store",
    "PUT - default graph",
    "GET of PUT - default graph",
    "PUT - mismatched payload",
    "PUT - empty graph",
    "GET of PUT - empty graph",
    "DELETE - existing graph",
    "GET of DELETE - existing graph",
    "DELETE - non-existent graph",
    "POST - existing graph",
    "GET of POST - existing graph",
    "POST - multipart/form-data",
    "GET of POST - multipart/form-data",
    "POST - create  new graph",
    "GET of POST - create  new graph",
    "POST - empty graph to existing graph",
    "GET of POST - after noop",
    "HEAD on an existing graph",
    "HEAD on a non-existing graph",
]

GRAPH1=\
u"""<?xml version="1.0"?>
<rdf:RDF
    xmlns:rdf ="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
    xmlns:foaf="http://xmlns.com/foaf/0.1/"
    xmlns:v   ="http://www.w3.org/2006/vcard/ns#"
    >
    <foaf:Person rdf:about="%s">
        <foaf:businessCard>
            <v:VCard>
                <v:fn>%s</v:fn>
            </v:VCard>
        </foaf:businessCard>
    </foaf:Person>
</rdf:RDF>"""

GRAPH2=\
u"""<?xml version="1.0"?>
<rdf:RDF
    xmlns:rdf ="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
    xmlns:foaf="http://xmlns.com/foaf/0.1/"
    xmlns:v   ="http://www.w3.org/2006/vcard/ns#"
    >
    <foaf:Person rdf:about="%s">
        <foaf:businessCard>
            <v:VCard>
                <v:given-name>%s</v:given-name>
            </v:VCard>
        </foaf:businessCard>
    </foaf:Person>
</rdf:RDF>"""

GRAPH3=\
u"""<?xml version="1.0"?>
<rdf:RDF
    xmlns:rdf ="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
    xmlns:foaf="http://xmlns.com/foaf/0.1/"
    xmlns:v   ="http://www.w3.org/2006/vcard/ns#"
    >
    <rdf:Description rdf:about="%s">
        <foaf:name>%s</foaf:name>
    </rdf:Description>
</rdf:RDF>"""

GRAPH4=\
u"""<?xml version="1.0"?>
<rdf:RDF
    xmlns:rdf ="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
    xmlns:foaf="http://xmlns.com/foaf/0.1/"
    xmlns:v   ="http://www.w3.org/2006/vcard/ns#"
    >
    <foaf:Person rdf:about="%s">
        <foaf:name>%s</foaf:name>
        <foaf:businessCard>
            <v:VCard>
                <v:fn>%s</v:fn>
            </v:VCard>
        </foaf:businessCard>
    </foaf:Person>
</rdf:RDF>"""

GRAPH5=\
u"""<?xml version="1.0"?>
<rdf:RDF
    xmlns:rdf ="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
    xmlns:foaf="http://xmlns.com/foaf/0.1/"
    xmlns:v   ="http://www.w3.org/2006/vcard/ns#"
    >
    <rdf:Description rdf:about="%s">
        <foaf:familyName>%s</foaf:familyName>
    </rdf:Description>
</rdf:RDF>
"""

GRAPH6=\
u"""<?xml version="1.0"?>
<rdf:RDF
    xmlns:rdf ="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
    xmlns:foaf="http://xmlns.com/foaf/0.1/"
    xmlns:v   ="http://www.w3.org/2006/vcard/ns#"
    >
    <rdf:Description rdf:about="%s">
        <foaf:givenName>%s</foaf:givenName>
    </rdf:Description>
</rdf:RDF>
"""

GRAPH7=\
u"""<?xml version="1.0"?>
<rdf:RDF
    xmlns:rdf ="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
    xmlns:foaf="http://xmlns.com/foaf/0.1/"
    xmlns:v   ="http://www.w3.org/2006/vcard/ns#"
    >
    <foaf:Person rdf:about="%s">
        <foaf:name>%s</foaf:name>
        <foaf:givenName>Jane</foaf:givenName>
        <foaf:familyName>Doe</foaf:familyName>
        <foaf:businessCard>
            <v:VCard>
                <v:fn>%s</v:fn>
            </v:VCard>
        </foaf:businessCard>
    </foaf:Person>
</rdf:RDF>"""

TEST_GRAPHS = {
    "PUT - Initial state"                   : (GRAPH1,"person/1",'xml',"John Doe"),
    "PUT - graph already in store"          : (GRAPH1,"person/1",'xml',"Jane Doe"),
    "PUT - default graph"                   : (GRAPH2,"",'xml',"Alice"),
    "PUT - mismatched payload"              : (GRAPH1,"person/1",'xml',"Jane Doe"),
    "PUT - empty graph"                     : (None,None,None,None),
    "PUT - replace empty graph"             : (GRAPH2,"",'xml',"Alice"),
    "POST - existing graph"                 : (GRAPH3,"person/1","xml","Jane Doe"),
    "HEAD on an existing graph"             : (GRAPH1,"person/1",'xml',"John Doe"),
    "HEAD on a non-existing graph"          : (None,None,None,None),
    "GET of POST - existing graph"          : (GRAPH4,"person/1","xml",("Jane Doe",)*2),
    "POST - create  new graph"              : (GRAPH2,"",'xml',"Alice"),
    "POST - empty graph to existing graph"  : (None,None,None,None),
    "multipart/form-data graph 1"           : (GRAPH5,"person/1",'xml',"Doe"),
    "multipart/form-data graph 2"           : (GRAPH6,"person/1",'xml',"Jane"),
    "GET of POST - multipart/form-data"     : (GRAPH7,"person/1",'xml',("Jane Doe",)*2)
}

class GraphStoreValidator(object):
    def __init__(self,graph_store_url,gs_url_internal):
        self.gs_url_internal = gs_url_internal
        if gs_url_internal:
            self.gs_url_internal = gs_url_internal\
            if gs_url_internal[-1] == '/' else gs_url_internal + '/'
        self.graph_store_url = graph_store_url
        self.graphs = TEST_GRAPHS.copy()
        self.graph_store_url_base = graph_store_url \
            if graph_store_url[-1] == '/' else graph_store_url + '/'

        for testName,(src,relUrl,format,name) in list(TEST_GRAPHS.items()):
            if src is None:
                self.graphs[testName] = None
            else:
                graphIri = URIRef(iri.absolutize(relUrl,self.graph_store_url_base))
                if isinstance(name,tuple):
                    params = (graphIri,)+name
                    src = src%params
                else:
                    src = src%(graphIri,name)
                self.graphs[testName] = IsomorphicTestableGraph().parse(StringIO(src),
                                                               format=format)

    def yieldResultElem(self,testName,successful,url,message=None):
        path_and_query = urlparse(url).path
        path_and_query = path_and_query + u'?' + urlparse(url
            ).query if urlparse(url).query else path_and_query
        attrs = {
            u'path'   : path_and_query,
            u'name'   : testName,
            u'result' : u'passed' if successful else u'failed'
        }

        assert testName in TESTS, testName
        testId = testName.replace(' ','_').replace('-','').replace('/','_').lower()
        attrs['id'] = testId

        if message:
            return E(
                (TEST_NS,u'Result'),attrs,message
            )
        else:
            return E(
                (TEST_NS,u'Result'),attrs
            )

    def graphSubmit(
            self,
            h,
            url,
            testName,
            getTestName=None,
            expectedStatus=[201,204],
            imt='text/turtle; charset=utf-8',
            format='n3',
            method='PUT',
            responseInfo=None,
            getUrl=None):
        responseInfo = responseInfo if responseInfo is not None else []
        graph = self.graphs[testName]
        body = graph.serialize(format=format) if graph is not None else u""
        hasPayload = False
        headers = {'cache-control'  : 'no-cache'}
        if method in ['PUT','POST']:
            hasPayload = True
            headers['content-type']   = imt
            headers['content-length'] = str(len(body))
        resp, content = h.request(
            url,
            method,
            body=body if hasPayload else None,
            headers=headers
        )
        responseInfo.append((resp,content))

        if isinstance(expectedStatus,list):
            matchingStatus = resp.status in expectedStatus
        else:
            matchingStatus = resp.status == expectedStatus

        if method == 'HEAD':
            if content:
                yield self.yieldResultElem(
                    testName,
                    False,
                    url,
                    u'HEAD response should have no content in the body'
                )
            elif not matchingStatus:
                yield self.yieldResultElem(
                    testName,
                    False,
                    url,
                    u'expected status %s, received %s (%s)'%(
                        expectedStatus,
                        resp.status,
                        content
                        )
                )
            elif 'content-length' not in resp:
                yield self.yieldResultElem(
                    testName,
                    False,
                    url,
                    u'expected content-length header in response'
                )
            elif 'content-type' not in resp:
                yield self.yieldResultElem(
                    testName,
                    False,
                    url,
                    u'expected content-type header in response'
                )
            else:
                yield self.yieldResultElem(testName,True,url)
        elif not matchingStatus:
            yield self.yieldResultElem(
                testName,
                False,
                url,
                u'expected status %s, received %s (%s)'%(
                    expectedStatus,
                    resp.status,
                    content
                )
            )
        else:
            yield self.yieldResultElem(testName,True,url)
        if getTestName:
            _url = getUrl if getUrl else url
            for el in self.isomorphCheck(testName,h,_url,getTestName):
                yield el

    def isomorphCheck(self,testName,h,url,alternativeTestName=None):
        try:
            resp, content = h.request(url,"GET",headers={
                'cache-control': 'no-cache',
                'Accept'       : 'text/turtle; charset=utf-8'
            })
        except:
            yield self.yieldResultElem(
                testName,
                False,
                url,
                u'Unable to perform content negotiated GET'%repr(e)
            )
        else:
            getTestName = alternativeTestName if alternativeTestName else testName
            if resp.status == 200:
                if 'content-type' not in resp or resp['content-type'].find('text/turtle')+1:
                    if content is not None and not content.strip():
                        content = None
                    g1=IsomorphicTestableGraph().parse(
                        StringIO(content),
                        format='n3') if content is not None else None
                    if g1 != self.graphs[testName]:
                        print "Unexpected response: "
                        print content
                        print resp.status
                        print resp
                        print "----"*5
                        yield self.yieldResultElem(
                            getTestName,
                            False,
                            url,
                            u'unexpected returned RDF graph'
                        )
                    else:
                        yield self.yieldResultElem(getTestName,True,url)
                elif 'content-type' in resp:
                    yield self.yieldResultElem(
                        getTestName,
                        False,
                        url,
                        u'expected returned content-type of "text/turtle; charset=utf-8", received %s'%(
                            resp['content-type']
                            )
                    )
            else:
                yield self.yieldResultElem(
                    getTestName,
                    False,
                    url,
                    u'expected status %s, received %s (%s)'%(
                        200,resp.status,content
                        )
                )

    def runTests(self):
        h = httplib2.Http()
        url          = iri.absolutize("person/1.ttl",self.graph_store_url_base)
        if self.gs_url_internal:
            internal_url = iri.absolutize("person/1.ttl",self.gs_url_internal)
            indirect_url = self.graph_store_url_base + '?graph=' + urllib2.quote(internal_url)
        else:
            indirect_url = self.graph_store_url_base + '?graph=' + urllib2.quote(url)

        for el in self.graphSubmit(h,url,"PUT - Initial state","GET of PUT - Initial state"):
            yield el

        h = httplib2.Http()
        url      = iri.absolutize("person/1.ttl",self.graph_store_url_base)
        testName = "HEAD on an existing graph"
        for el in self.graphSubmit(h,url,testName,expectedStatus=[200,204],method='HEAD'):
            yield el

        h = httplib2.Http()
        testName = "HEAD on a non-existing graph"
        resp, content = h.request(
            iri.absolutize("person/4.ttl",
                self.graph_store_url_base),
            "HEAD"
        )
        if resp.status == 404:
            yield self.yieldResultElem(testName,True,url)
        else:
            yield self.yieldResultElem(
                testName,
                False,
                url,
                u'expected status %s, received %s (%s)'%(
                    404,
                    resp.status,
                    content
                    )
            )

        testName = u"PUT - graph already in store"
        for el in self.graphSubmit(
                            h,
                            url,
                            testName,
                            expectedStatus=[200,204],
                            getTestName="GET of PUT - graph already in store",
                            getUrl=indirect_url):
            yield el

        url = self.graph_store_url_base+'?default'
        testName = "PUT - default graph"
        for el in self.graphSubmit(
                            h,
                            url,
                            testName,
                            "GET of PUT - default graph",
                            expectedStatus=204):
            yield el

        h = httplib2.Http()
        url      = iri.absolutize("person/1.ttl",self.graph_store_url_base)
        testName = "PUT - mismatched payload"
        for el in self.graphSubmit(h,url,testName,expectedStatus=400,imt='application/rdf+xml'):
            yield el

        h = httplib2.Http()
        url          = iri.absolutize("person/2.ttl",self.graph_store_url_base)
        if self.gs_url_internal:
            internal_url = iri.absolutize("person/2.ttl",self.gs_url_internal)
            indirect_url = self.graph_store_url_base + '?graph=' + urllib2.quote(internal_url)
        else:
            indirect_url = self.graph_store_url_base + '?graph=' + urllib2.quote(url)

        for el in self.graphSubmit(
                            h,
                            indirect_url,
                            "PUT - empty graph",
                            "GET of PUT - empty graph",
                            getUrl=url):
            yield el

        h = httplib2.Http()
        url      = iri.absolutize("person/2.ttl",self.graph_store_url_base)
        for el in self.graphSubmit(
            h,
            url,
            "PUT - replace empty graph",
            "GET of replacement for empty graph",
            expectedStatus=[201,204]):
            yield el

        testName = "DELETE - existing graph"
        resp, content = h.request(url,"DELETE")
        if resp.status not in  [200,204]:
            yield self.yieldResultElem(
                testName,
                False,
                url,
                u'expected status %s, received %s (%s)'%(
                    200,
                    resp.status,
                    content
                    )
            )
        else:
            yield self.yieldResultElem(testName,True,url)

        testName = "GET of DELETE - existing graph"
        resp, content = h.request(url,"GET",headers={
            'cache-control': 'no-cache',
        })

        if resp.status == 404:
            yield self.yieldResultElem(testName,True,url)
        else:
            yield self.yieldResultElem(
                testName,
                False,
                url,
                u'expected status %s, received %s (%s)'%(
                    404,
                    resp.status,
                    content
                    )
            )

        testName = "DELETE - non-existent graph"
        resp, content = h.request(url,"DELETE")
        if resp.status == 404:
            yield self.yieldResultElem(testName,True,url)
        else:
            yield self.yieldResultElem(
                testName,
                False,
                url,
                u'expected status %s, received %s (%s)'%(
                    404,
                    resp.status,
                    content
                    )
            )

        h = httplib2.Http()
        url      = iri.absolutize("person/1.ttl",self.graph_store_url_base)
        testName = "POST - existing graph"
        for el in self.graphSubmit(
            h,
            url,
            testName,
            expectedStatus=[200,204],
            method='POST'):
            yield el
        for el in self.isomorphCheck(
            "GET of POST - existing graph",
            h,
            url,
            alternativeTestName="GET of POST - existing graph"):
            yield el

        try:
            from poster.encode import multipart_encode, MultipartParam
            from poster.streaminghttp import register_openers
            h = httplib2.Http()
            testName = "POST - multipart/form-data"

            register_openers()

            datagen, headers = multipart_encode(
                [
                    MultipartParam(
                        "lastName.ttl",
                        self.graphs["multipart/form-data graph 1"].serialize(
                            format='turtle'
                        ),
                        filename="lastName.ttl",
                        filetype='text/turtle; charset=utf-8'
                    ),
                    MultipartParam(
                        "firstName.ttl",
                        self.graphs["multipart/form-data graph 2"].serialize(
                            format='turtle'
                        ),
                        filename="firstName.ttl",
                        filetype='text/turtle; charset=utf-8'
                    )
                ]
            )

            req = urllib2.Request(url, datagen, headers)
            resp = urllib2.urlopen(req)

            if resp.code not in [200,204]:
                yield self.yieldResultElem(
                    testName,
                    False,
                    url,
                    u'expected status %s, received %s (%s)'%(
                        [200,204],
                        resp.code,
                        resp.read()
                    )
                )
            else:
                yield self.yieldResultElem(testName,True,url)
            for el in self.isomorphCheck(
                            "GET of POST - multipart/form-data",
                            h,
                            url,
                            "GET of POST - multipart/form-data"):
                yield el
        except urllib2.HTTPError, e:
            yield self.yieldResultElem(
                testName,
                False,
                url,
                u'Invalid Server response: %s'%repr(e)
            )
        except ImportError:
            pass

        h = httplib2.Http()
        url      = self.graph_store_url_base
        testName = "POST - create  new graph"
        responseInfo = []
        for el in self.graphSubmit(
            h,
            url,
            testName,
            expectedStatus=201,
            method='POST',
            responseInfo=responseInfo):
            yield el
        if responseInfo:
            resp,content = responseInfo[0]
            if 'location' in resp:
                yield self.yieldResultElem(testName,True,url)
                url = resp['location']
                for el in self.isomorphCheck(
                    "POST - create  new graph",
                    h,
                    url,
                    alternativeTestName="GET of POST - create  new graph"):
                    yield el

                h = httplib2.Http()
                for el in self.graphSubmit(
                    h,
                    url,
                    "POST - empty graph to existing graph",
                    expectedStatus=[200,204],
                    method='POST'):
                    yield el

                for el in self.isomorphCheck(
                    "POST - create  new graph",
                    h,
                    url,
                    alternativeTestName="GET of POST - after noop"):
                    yield el
            else:
                yield self.yieldResultElem(
                    testName,
                    False,
                    url,
                    u'POST to graph store should return Location header: %s'%repr(resp)
                )

class WsgiGSPValidator(object):
    def __init__(self,app): pass
    def __call__(self, environ, start_response):
        req = Request(environ)
        if req.method == 'POST':
            if req.content_type == 'application/x-www-form-urlencoded':
                implementation_url = req.POST.get('doap_project_url')
                validator_url      = req.POST.get('gs_url')
                project_name       = req.POST.get('doap_project_name')
                gsInternalUrl      = req.POST.get('gs_url_internal')
            else:
                form = cgi.FieldStorage(
                    fp=StringIO(req.body),
                    environ=request.environ
                )
                implementation_url = form.getvalue("doap_project_url")
                validator_url      = form.getvalue("gs_url")
                project_name       = form.getvalue('doap_project_name')
                gsInternalUrl      = form.getvalue('gs_url_internal')
        elif req.method == 'GET':
            implementation_url = req.params.get('doap_project_url')
            validator_url      = req.params.get("gs_url")
            project_name       = req.params.get('doap_project_name')
            gsInternalUrl      = req.params.get('gs_url_internal')
        else:
            rt = "Validation HTTP methods supported are POST and GET: received %s"%(
                req.method
            )
            start_response("405 Method Not Allowed",
                [("Content-Length",  len(rt))])
            return rt
        if validator_url and implementation_url:
            validator = GraphStoreValidator(validator_url,gsInternalUrl)
            src = StringIO()
            w = structwriter(indent=u"yes", stream=src)
            w.feed(
                ROOT(
                    E(
                        (TEST_NS,u'Results'),
                        (elem for elem in validator.runTests())
                    )
                )
            )
            requestedRDF = set(layercake_mimetypes).intersection(req.accept)
            if 'HTTP_ACCEPT' not in environ or not requestedRDF:
                rt = transform(
                    src.getvalue(),
                    os.path.join(
                        config().get('demo_path'),
                        'gsp_validation_results.xslt'),
                    params={
                        u'project'  : project_name,
                        u'url'      : implementation_url})
                start_response("200 Ok",
                    [("Content-Type"  , HTML_IMT+';charset=utf-8'),
                     ("Content-Length",  len(rt))])
                return rt
            elif requestedRDF:
                preferredMT = req.accept.best_match(layercake_mimetypes)
                format = layercake_mimetypes[preferredMT]
                rt = transform(
                    src.getvalue(),
                    os.path.join(
                        config().get('demo_path'),
                        'gsp_validation_results_earl.xslt'),
                    params={
                        u'project'  : project_name,
                        u'date'     : datetime.date.today().isoformat(),
                        u'url'      : implementation_url})
                g=Graph().parse(StringIO(rt),format='xml')
                g.bind('validator',URIRef('http://metacognition.info/gsp_validation/'))
                g.bind('gsp',URIRef('http://www.w3.org/2009/sparql/docs/tests/data-sparql11/http-rdf-update/manifest#'))
                g.bind('test',
                       URIRef(
                           'http://www.w3.org/2009/sparql/docs/tests/data-sparql11/http-rdf-update/#')
                )
                rt=g.serialize(format=format)
                start_response("200 Ok",
                    [("Content-Type"  , preferredMT),
                     ("Content-Length",  len(rt))])
                return rt
        else:
            msg="Bad request"
            start_response("303 See Other",
                [("Location","gsp.validator.form?message=Please+provide+a+Graph+Store+URL+to+validate+and+an+implementation+url"),
                 ("Content-Length",  len(msg))])
            return msg

@service(SERVICE_ID, 'gsp.validator.run',wsgi_wrapper=WsgiGSPValidator)
def validation():pass


import sys
from StringIO import StringIO
from webob import Request
from akamu.security.acl    import web_acl, WEB_ACL_NS
from akamu.security.repoze import RepozeWrapper
from akara.services        import service, simple_service
from rdflib import plugin, URIRef, OWL, RDFS, RDF, Namespace

ACCESS_NS = Namespace('http://example.com/access_classes/')

SERVICE_ID_PREFIX = 'http://example.com/service/'

def CreateRepozeMiddleWare(app):
    from repoze.who.plugins.htpasswd import HTPasswdPlugin
    from repoze.who.plugins.basicauth import BasicAuthPlugin
    from repoze.who.plugins.auth_tkt import AuthTktCookiePlugin
    from repoze.who.plugins.redirector import RedirectorPlugin
    from repoze.who.interfaces import IChallenger
    from repoze.who.middleware import PluggableAuthenticationMiddleware
    io = StringIO()
    io.write('admin:admin')
    io.seek(0)
    def cleartext_check(password, hashed):
        return password == hashed
    htpasswd = HTPasswdPlugin(io, cleartext_check)
    basicauth = BasicAuthPlugin('repoze.who')
    redirector = RedirectorPlugin('/login')
    redirector.classifications = {IChallenger:['browser'],} # only for browser
    identifiers = [('basicauth', basicauth)]
    authenticators = [('htpasswd', htpasswd)]
    challengers = [('redirector', redirector),
                   ('basicauth', basicauth)]
    mdproviders = []

    from repoze.who.classifiers import default_request_classifier
    from repoze.who.classifiers import default_challenge_decider
    import logging
    log_stream = sys.stdout

    return PluggableAuthenticationMiddleware(
        app,
        identifiers,
        authenticators,
        challengers,
        mdproviders,
        default_request_classifier,
        default_challenge_decider,
        log_stream = log_stream,
        log_level = logging.DEBUG
    )


def service_stub(environ, start_response, supported_methods):
    req = Request(environ)
    if req.method not in supported_methods:
        start_response("405 Method Not Allowed", [])
        return "Method not allowed for this resource"
    else:
        rt = 'Success'
        start_response("200 Ok",
                       [("Content-Type", 'text/plain'),
                        ("Content-Length", len(rt))]
        )
        return rt


@service(SERVICE_ID_PREFIX+'1','service.1',wsgi_wrapper=CreateRepozeMiddleWare)
@web_acl('mysqlDataset',simple_service=False)
def service_1(environ, start_response):
    return service_stub(
        environ,
        start_response,
        ['POST','GET']
    )

@service(SERVICE_ID_PREFIX + '2', 'service.2',wsgi_wrapper=CreateRepozeMiddleWare)
@web_acl('mysqlDataset',
         accessMap = { "POST" : WEB_ACL_NS.Append },
         simple_service=False)
def service_2(environ, start_response):
    return service_stub(
                environ,
                start_response,
                ['POST','GET','PUT']
    )

@simple_service(
    'GET' ,
    SERVICE_ID_PREFIX + '3',
    'service.3',
    'text/plain',
    wsgi_wrapper=CreateRepozeMiddleWare)
@web_acl('mysqlDataset')
def service_3(): return "Success"